/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/map_utils.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "bnd/discosrv.hpp"

namespace jabber
{
  void DiscoSrv::CreateNode(const JabUri& uri, const JabUri& parent)
  {
    if(uri.IsNull() || !uri.IsValid()){
      OAPR_THROW(oapr::invalid_arg,"uri is invalid");
    }
    node_elem node(new DiscoNode);
    node->Assign(uri.Jid());
    if(uri.HasParameter("node")){
      node->NodeName(uri.Parameter("node"));
    }
    if(uri.HasParameter("name")){
      node->DisplayName(uri.Parameter("name"));
    }

    if(!parent.IsNull()){
      /* check to see if the parent exists! */
      if(m_nodes.find(parent)==m_nodes.end()){
	OAPR_THROW(oapr::invalid_arg,"parent uri does not exist");
      }
      m_children.insert(std::make_pair(parent,node));
    }
    m_nodes.insert(std::make_pair(uri,node));
  }
  DiscoNode*const DiscoSrv::GetNode(const JabUri& uri)const
  {
    node_cont::const_iterator fnd=m_nodes.find(uri);
    if(fnd==m_nodes.end()){
      std::string msg=uri.Uri()+" does not exist";
      OAPR_THROW(oapr::invalid_arg,msg);
    }else{
      return fnd->second.get();
    }
  }
  void DiscoSrv::Add(const JabUri& uri, const DiscoIdentity& ident)
  {
    DiscoNode*const node=GetNode(uri);
    node->AddIdentity(ident);
  }
  void DiscoSrv::Add(const JabUri& uri, const DiscoFeature& feature)
  {
    DiscoNode*const node=GetNode(uri);
    node->AddFeature(feature);
  }

  void DiscoSrv::Remove(const JabUri& uri, const DiscoIdentity& ident)
  {
    DiscoNode*const node=GetNode(uri);
    node->RemoveIdentity(ident);
  }
  void DiscoSrv::Remove(const JabUri& uri, const DiscoFeature& feature)
  {
    DiscoNode*const node=GetNode(uri);
    node->RemoveFeature(feature);
  }

  JabDiscoItem* DiscoSrv::Response(const JabDiscoItem& jdi)const
  {
    std::auto_ptr<JabDiscoItem> rv(new JabDiscoItem);
    typedef std::list<node_elem> node_list_type;
    node_list_type nodeList;

    JabUri uri = jdi.NodeUri();
    /* no quick shortcuts here, damnit*/
    child_cont::const_iterator lowpos=m_children.lower_bound(uri);
    child_cont::const_iterator hipos=m_children.upper_bound(uri);
    for(;lowpos!=hipos;++lowpos){
      nodeList.push_back(lowpos->second);
    }

    if(!nodeList.empty()){
      /* if we have items, then insert them */
      node_list_type::const_iterator ndPos=nodeList.begin();
      JabDiscoItem::Item item;
      JabUri nodeUri;
      for(;ndPos!=nodeList.end();++ndPos){
	/* translate the node uri to a item*/
	nodeUri = (*ndPos)->Uri();
	item.Jid(nodeUri.Jid());
	if(nodeUri.HasParameter("node")){
	  item.Node(nodeUri.Parameter("node"));
	}
	if(nodeUri.HasParameter("name")){
	  item.Name(nodeUri.Parameter("name"));
	}

	rv->Insert(item);
      }
    }

    /* adjust the to from, type and id*/
    rv->To(jdi.From());
    rv->From(jdi.To());
    rv->Type("result");
    rv->Id(jdi.Id());

    
    return rv.release();
  }
  JabDiscoInfo* DiscoSrv::Response(const JabDiscoInfo& jdi)const
  {
    JabUri tempUri = jdi.NodeUri();

    /* find the node in the list*/

    node_cont::const_iterator fnd=m_nodes.find(tempUri);
    if(fnd==m_nodes.end()){
      /* we didn't find antyhing... should we throw here?*/
      return 0;
    }

    return fnd->second->GetInfo();    
  }


};
