/********************************************************************
 * OAbnd
 * lib to provide services for browse, disco and feature negotation
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <algorithm>
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xmlutil.hpp"
#include "bnd/ijabdiscoinfo.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  
  JabUri JabDiscoInfo::NodeUri()const
  {
    JabUri rv;
    JabJid jid;
    if(Type()=="get"){
      jid=To();
    }else if(Type()=="result"){
      jid=From();
    }else{
      return rv;
    }
    rv.Query("disco");
    rv.Jid(jid);
    if(!m_nodeName.empty()){
      rv.Parameter("node",m_nodeName);
    }
    return rv;
  }
  JabDiscoInfo::identity_list JabDiscoInfo::Identities()const
  {
    JabDiscoInfo::identity_list rv;
    std::copy(m_identities.begin(),m_identities.end(),
	      std::back_inserter(rv));
    return rv;
  }
  JabDiscoInfo::feature_list JabDiscoInfo::Features()const
  {
    JabDiscoInfo::feature_list rv;
    std::copy(m_features.begin(),m_features.end(),
	      std::back_inserter(rv));
    return rv;
  }
  void JabDiscoInfo::Insert(const Identity& id)
  {
    if(Exists(id)){return;}
    m_identities.push_back(id);
  }
  void JabDiscoInfo::Insert(const Feature& fe)
  {
    if(Exists(fe)){return;}
    m_features.push_back(fe);
  }
  void JabDiscoInfo::Remove(const Identity& id)
  {
    identity_cont::iterator fnd=std::find(m_identities.begin(),
					  m_identities.end(),
					  id);
    if(fnd != m_identities.end()){
      m_identities.erase(fnd);
    }
  }
  void JabDiscoInfo::Remove(const Feature& fe)
  {
    feature_cont::iterator fnd=std::find(m_features.begin(),
					 m_features.end(),
					 fe);
    if(fnd!=m_features.end()){
      m_features.erase(fnd);
    }					 
  }
  bool JabDiscoInfo::Exists(const Identity& id)const
  {
    identity_cont::const_iterator fnd=std::find(m_identities.begin(),
					  m_identities.end(),
					  id);
    if(fnd!=m_identities.end()){
      return true;
    }
    return false;
  }
  bool JabDiscoInfo::Exists(const Feature& fe)const
  {
    feature_cont::const_iterator fnd=std::find(m_features.begin(),
					       m_features.end(),
					       fe);
    if(fnd!=m_features.end()){
      return true;
    }
    return false;
  }
  DOMNode*const JabDiscoInfo::BuildXml()
  {

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
    oa::dom_ptr<DOMElement> queryNode;
    {
      oa::xmlstr<XMLCh,char> queryName("query"),
	queryURI(InfoUri()),
	xmlnsName("xmlns");

      /*create the query element in ns*/
      queryNode.reset(doc->createElementNS(queryURI,queryName));
      /* force the xmlns to print on the formatter*/
      oa::XercesXmlns()(queryNode.get(),queryURI);
      //      queryNode->setAttribute(xmlnsName.get(),queryURI.get());
    }
    /* if we have a node, set it as an attribute */
    if(!m_nodeName.empty()){
      oa::xmlstr<XMLCh,char> nodeName("node"),
	nodeValue(m_nodeName.c_str());
      queryNode->setAttribute(nodeName,nodeValue);
    }
    
    /* if we are a result, then build idents and features */
    if(Type()=="result"){
      BuildIdentities(queryNode.get());
      BuildFeatures(queryNode.get());
    }

    /* now build the iq and append the query*/
    oa::dom_ptr<DOMNode> iq(BuildIq());
    iq->appendChild(queryNode.release());
    return iq.release();
  }
  void JabDiscoInfo::BuildIdentities(DOMNode*const qry)
  {
    DOMDocument *doc = qry->getOwnerDocument();
    oa::xmlstr<XMLCh,char> identityName("identity"),
      catName("category"),
      typeName("type"),
      nameName("name"),
      catValue,typeValue,nameValue;
    
    identity_cont::iterator pos=m_identities.begin();
    for(;pos!=m_identities.end();++pos){
      DOMElement *identity=doc->createElement(identityName.get());
      if(!pos->Category().empty()){
	catValue.set(pos->Category().c_str());
	identity->setAttribute(catName,catValue);
      }
      if(!pos->Name().empty()){
	nameValue.set(pos->Name().c_str());
	identity->setAttribute(nameName,nameValue);
      }
      if(!pos->Type().empty()){
	typeValue.set(pos->Type().c_str());
	identity->setAttribute(typeName,typeValue);
      }
      qry->appendChild(identity);
    }
  }
  void JabDiscoInfo::BuildFeatures(DOMNode*const qry)
  {
    DOMDocument *doc = qry->getOwnerDocument();
    oa::xmlstr<XMLCh,char> featureName("feature"),
      varName("var"),
      varValue;

    feature_cont::iterator pos=m_features.begin();
    for(;pos!=m_features.end();++pos){
      if(!pos->Var().empty()){
	DOMElement* feature=doc->createElement(featureName);
	varValue.set(pos->Var().c_str());
	feature->setAttribute(varName,varValue);
	qry->appendChild(feature);
      }
    }    
  }
};
