/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <iostream>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOMText.hpp>

#include "framework/dombuilder.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
XERCES_CPP_NAMESPACE_USE
namespace oa
{

  // --------------------------------------
  // Handlers for the SAX DocumentHandler interface
  //--------------------------------------
  void DomBuilder::startDocument()
  {
    /* This does absolutely nothing.
     * we need the uri and root element to start up
     * a document. So the startElement() will deal
     * with this
     */
    
  }
  void DomBuilder::startElement(const XMLCh *const uri, 
				const XMLCh *const localname,
				const XMLCh *const qname,
				const Attributes& attributes)
  {
    try
    {
      oa::xmlstr<char,XMLCh> suri(uri),slocalname(localname),
	  sqname(qname);
      bool docCreated(false);
    
      /* first, do we have a document?*/
      if(!m_doc.get()){
	/* create it from our friend xmlutil*/
	
	m_doc.reset(oa::XMLUtils().Document(uri,qname,true));	
	m_root = m_doc->getDocumentElement();
	m_position=m_root;
	m_current=m_root;
	docCreated=true;

	//force the default ns to exist
	if(m_defaultNs.get()){
	  oa::xmlstr<XMLCh,char> nm("xmlns");
	  ((DOMElement*)m_root)->setAttribute(nm,m_defaultNs.get());
	}
      }
    
      /* should we do anything with the text node */
      if(m_text && m_current){
	m_current->appendChild(m_text);/*adding to the current node*/
	m_text=0;/*nulling the text since m_current now owns it*/
      }
    
      /* should we append current node */
      if(m_append && m_position){
	m_position->appendChild(m_current);
	m_position=m_current;
	m_append=false;
      }
    
      if(!docCreated){
	/* now we finally get around to the creation*/
	m_current = m_doc->createElementNS(uri,qname);
	m_append=true;//<-- is this necessary?
      }

      
      /* deal with attributes*/
      for(unsigned int attrI=0; attrI<attributes.getLength();++attrI){
	const XMLCh*const aUri=attributes.getURI(attrI);
	const XMLCh*const aQname=attributes.getQName(attrI);
	const XMLCh*const aValue=attributes.getValue(attrI);

	oa::xmlstr<char,XMLCh> auri(aUri),name(aQname),value(aValue);
	
	DOMAttr* attr = m_doc->createAttributeNS(aUri,aQname);
	attr->setValue(aValue);
	((DOMElement*)m_current)->setAttributeNode(attr);
      }
    }catch(const XMLException &e){
      oa::xmlstr<char,XMLCh> emsg(e.getMessage());
      std::cerr<<"XML ERROR: "<<emsg<<"\n";
    }catch(const DOMException &e){
      oa::xmlstr<char,XMLCh> emsg(e.msg);
      std::cerr<<"DOM ERROR: "<<emsg<<"\n";
    }
  }
  void DomBuilder::startPrefixMapping(const XMLCh *const prefix,
				      const XMLCh *const uri)
  {
    if(m_doc.get() || m_defaultNs.get()){return;}
    if(XMLString::stringLen(prefix)<=0 &&
       XMLString::stringLen(uri)>0){
      m_defaultNs.reset(XMLString::replicate(uri));
    }


  }
  void DomBuilder::characters(const XMLCh* const chars,
			      const unsigned int length)
  {
    /* if we don't have anything, or anywhere to put it,
     * simply return*/
    if(!chars || !m_current){return;}

    if(!m_text){
      m_text = m_doc->createTextNode(chars);
    }else{
      m_text->appendData(chars);
    }
  }
  void DomBuilder::ignorableWhitespace(const XMLCh *const chars,
				       const unsigned int length)
  {
  }
  void DomBuilder::processingInstruction(const XMLCh *const target,
					 const XMLCh *const data)
  {

  }
  void DomBuilder::setDocumentLocator(const Locator *const locator)
  {
  }

  void DomBuilder::endPrefixMapping(const XMLCh *const prefix)
  {
  }
  void DomBuilder::endElement(const XMLCh *const uri,
			      const XMLCh *const localname,
			      const XMLCh *const qname)
  {
    DOMNode *parent=0;
    /* do we have anything to end?*/
    if(!m_position){return;}


    /* take care of misc dangling nodes*/

    /* text node first*/
    if(m_text && m_current){
      m_current->appendChild(m_text);
      m_text=0;
    }
    
    if(m_append && m_position){
      m_position->appendChild(m_current);
      parent=m_position;
      m_append=false;
    }else if(m_position){
      parent = m_position->getParentNode();
      m_append=false;
    }

    if(!parent || parent->getNodeType()==DOMNode::DOCUMENT_FRAGMENT_NODE){
      return;
    }
    m_position=parent;
  }
  void DomBuilder::endDocument()
  {
  }
  void DomBuilder::skippedEntity(const XMLCh *const name)
  {
  }



  // --------------------------------------
  // Functions from LexicalHandler interface
  // --------------------------------------
  void DomBuilder::startCDATA()
  {
    /* wonder if this will *ever* be called*/
  }
  void DomBuilder::startEntity(const XMLCh* const name)
  {
  }
  void DomBuilder::startDTD(const XMLCh *const name, 
			    const XMLCh *const publicId,
			    const XMLCh *const systemId)
  {
  }
  void DomBuilder::comment(const XMLCh *const chars, 
			   const unsigned int length)
  {
  }
  void DomBuilder::endDTD()
  {
  }
  void DomBuilder::endEntity(const XMLCh *const name)
  {
  }
  void DomBuilder::endCDATA()
  {
  }
  
  // --------------------------------------
  // Functions for the SAX DTDHandler interface
  //--------------------------------------
  void DomBuilder::notationDecl(const XMLCh *const name,
				const XMLCh *const publicId,
				const XMLCh *const systemId)
  {
  }
  void DomBuilder::unparsedEntityDecl(const XMLCh *const name,
				      const XMLCh *const publicId,
				      const XMLCh *const systemId,
				      const XMLCh *const notationName)
  {
  }
  void DomBuilder::resetDocType()
  {
  }

  // --------------------------------------
  // Functions for the SAX EntifyResolver interface
  //--------------------------------------
  InputSource* DomBuilder::resolveEntity(const XMLCh *const publicId,
					 const XMLCh *const systemId)
  {
    return 0;
  }
  // --------------------------------------
  // Functions for the SAX ErrorHandler interface
  //--------------------------------------
  void DomBuilder::error(const SAXParseException& exception)
  {
  }
  void DomBuilder::fatalError(const SAXParseException& exception)
  {
  }
  void DomBuilder::warning(const SAXParseException& exception)
  {
  }
  void DomBuilder::resetErrors()
  {
  }

};
