/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
/**** --== Change Log ==-- ****
 * 2002-09-26
 * - The m_handle was being set to non-zero and if an error 
 *   occurred, then the dtor would try to close an invalid handle
 *   Changed the error catch to set m_handle to zero
 *
 */

#ifndef FRAMEWORK_LIB_LOADER_HPP
#define FRAMEWORK_LIB_LOADER_HPP
#include "framework/framework_exports.h"
#ifdef WIN32
#include <windows.h>
#else
#include <dlfcn.h>
#endif
#include <string>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
namespace oa
{
  template<class Result, class Func>
  class Loader
  {
  public:
    typedef Result* result_type;
    typedef Func factory_type;
  public:
    Loader(bool unload=false)
      : m_unload(unload),
	m_handle(0){}
    virtual ~Loader();
    result_type Load(std::string lib,std::string func);
    int Unload();
  private:
    bool m_unload;
    std::string m_lib;
    void *m_handle;
  };

#ifdef WIN32
  template<class Result, class Func>
  Loader<Result,Func>::~Loader(){
    if(m_handle && m_unload){
      FreeLibrary((HMODULE)m_handle);
    }
  }

  template<class Result, class Func>
  int Loader<Result,Func>::Unload(){
    return 1;
  }

  template<class Result, class Func>
  typename Loader<Result,Func>::result_type 
  Loader<Result,Func>::Load(std::string lib,std::string funcname)
  {
    m_handle = LoadLibraryEx(lib.c_str(),NULL,0);
    if(!m_handle){
      /**\todo put throw here*/
      return 0;
    }
    factory_type func = (factory_type)GetProcAddress((HMODULE)m_handle,funcname.c_str());
    if(!func){
      /**\todo put throw here*/
      return 0;
    }
    return (*func)();
  }
#else//put linux def here
  template<class Result, class Func>
  Loader<Result,Func>::~Loader(){
    if(m_handle && m_unload){
      dlclose(m_handle);
    }
  }
  template<class Result, class Func>
  int Loader<Result, Func>::Unload(){
    if(m_handle){
      int ret = dlclose(m_handle);
      m_handle = 0;
      return ret;
    }
    return 0;
  }



  template<class Result, class Func>
  typename Loader<Result,Func>::result_type 
  Loader<Result,Func>::Load(std::string lib,std::string funcname){
    char *err=0;
    //trying to load the lib
    m_handle = dlopen(lib.c_str(),RTLD_NOW | RTLD_GLOBAL);
    if(!m_handle){
      err = dlerror();
      m_handle=0;//<-- this will prevent from closing a dead handle
      /* throwing since there is not much else we can do here*/
      OAPR_THROW(oapr::runtime_error,err);
    }
    factory_type func = (factory_type)dlsym(m_handle,funcname.c_str());
    if(!func){
      err = dlerror();
      /* throwing since there is not much else we can do here*/
      OAPR_THROW(oapr::runtime_error,err);
    }
    return (*func)();
  }
  
#endif
};


#endif
