/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/

#include "first.h"
#include "openaether.hpp"
#include <fstream>
#include <sstream>
#include "boost/smart_ptr.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/parser_xml.hpp"

//apache includes for xml parsing
#include <xercesc/util/PlatformUtils.hpp> /*<-- for xmlinit and term*/
#include <xercesc/parsers/AbstractDOMParser.hpp>
#include <xercesc/dom/DOMImplementation.hpp>
#include <xercesc/dom/DOMImplementationLS.hpp>
#include <xercesc/dom/DOMImplementationRegistry.hpp>
#include <xercesc/dom/DOMBuilder.hpp>
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/framework/MemBufInputSource.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>

//following are for debugging purposes only
#include "framework/xmlutil.hpp"
#include <iostream>

XERCES_CPP_NAMESPACE_USE

namespace oa
{
  int XMLParser::Source(const std::string &src){
    m_file = src;
    return 0;
  }
  Config* XMLParser::Parse()
  {
    /**!todo create the error handler*/
    const XMLCh gLS[]={chLatin_L,chLatin_S, chNull };
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(gLS);
    dom_ptr<DOMBuilder> parser(impl->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS,0));

    /**!todo does this need to be configurable somehow?*/
    /*
    parser->setValidationScheme(AbstractDOMParser::Val_Auto);
    parser->setIncludeIgnorableWhitespace(false);
    parser->setDoNamespaces(false);
    parser->setDoSchema(false);
    parser->setValidationSchemaFullChecking(false);
    */
    DOMDocument *theDoc=0;
    try{
      boost::scoped_array<XMLCh> fileName(XMLString::transcode(m_file.c_str()));
      theDoc = parser->parseURI(m_file.c_str());
    }catch(...){
      /**!todo do something with this... ick*/
      return 0;
    }
    if(!theDoc){
      return 0;
    }

    if(!XercesHack(theDoc)){
      return 0;
    }

    Config *pcfg = Parse(theDoc->getDocumentElement());
    return pcfg;
  }

  ConfigChain* XMLParser::Parse(DOMNode *const nd)
  {

    std::auto_ptr<ConfigChain> pchain;//the chain that is to be returned
    std::auto_ptr<ConfigChain> pconfig;//the config returned from recursive parse
    std::string strNodeName;
    const XMLCh* ndName;//name of current node... used as key

    if(!nd){
      return 0;
    }
    if(nd->hasChildNodes() && nd->getNodeType() == DOMNode::ELEMENT_NODE){

      DOMNodeList *children = nd->getChildNodes();
      ndName = ((DOMElement*)nd)->getTagName();

      boost::scoped_array<char> cstrName(XMLString::transcode(ndName));
      pchain.reset(new ConfigChain(cstrName.get()));

      for(unsigned int i=0;i<children->getLength();++i){
	DOMNode* child = children->item(i);

	if(child->getNodeType() != DOMNode::ELEMENT_NODE){
	  continue;
	}
	DoAttrs(pchain.get(),child);
	ndName = ((DOMElement*)nd)->getTagName();
	if(IsKey(child)){
	  pconfig.reset(Parse(child));
	}else if(IsParam(child)){
	  XMLParser::strpr pr = GetParam(child);
	  if(!pr.first.empty()){
	    pchain->Set(pr.first,pr.second);
	  }
	}else{
	  continue;
	}
	//if we have a config insert it
	int temp=0;
	if(pconfig.get()){
	  temp = pchain->Insert(pconfig.release());
	}
	//always reset
	pconfig.reset();
      }      
    }
    return pchain.release();
  }
  bool XMLParser::IsKey(DOMNode *const nd)
  {
    /*
      Keys are basically elements with other keys or params as children
     */
    if(nd->hasChildNodes()){
      if(nd->getFirstChild()->getNodeType() != DOMNode::TEXT_NODE){
	return true;
      }
    }
    return false;
  }
  bool XMLParser::IsParam(DOMNode *const nd)
  {
    /*
      Two qualifications for param:
      - <elem>text</elem>
      - <elem/>
     */
    if(nd->hasChildNodes()){
      //it has a child node... is the first node a text node?
      if(nd->getFirstChild()->getNodeType() == DOMNode::TEXT_NODE){
	return true;
      }
    }else if(nd->getNodeType() == DOMNode::ELEMENT_NODE){
      //an empty element... view this as a switch
      return true;
    }
    //everything else is not a param
    return false;
  }
  XMLParser::strpr XMLParser::GetParam(DOMNode *const nd)
  {
    const XMLCh *ndName,*ndValue;
    boost::scoped_array<char> cstrName,cstrValue;
    //not verifying paramness... since it should be done previously
    ndName = ((DOMElement*)nd)->getTagName();
    DOMNode *child = nd->getFirstChild();
    if(child){
      ndValue = child->getNodeValue();
    }

    cstrName.reset(XMLString::transcode(ndName));
    cstrValue.reset(XMLString::transcode(ndValue));
    return std::make_pair(cstrName.get(),cstrValue.get());
			  
  }
  void XMLParser::DoAttrs(Config *cfg, DOMNode *const nd)
  {
    if(!cfg || !nd){
      return;
    }
    DOMNamedNodeMap *attrs = nd->getAttributes();
    if(attrs->getLength()<=0){
      return;
    }

    const XMLCh *aName,*aVal;
    boost::scoped_array<char> cstrName,cstrValue;
    for(unsigned int i=0; i<attrs->getLength(); ++i){
      DOMNode *attr = attrs->item(i);
      if(attr && attr->getNodeType() == DOMNode::ATTRIBUTE_NODE){
	aName=((DOMAttr *)attr)->getName();
	aVal=((DOMAttr *)attr)->getValue();
	cstrName.reset(XMLString::transcode(aName));
	cstrValue.reset(XMLString::transcode(aVal));
	cfg->Set(cstrName.get(),cstrValue.get());		 
      }
    }

  }

  int XMLParser::XercesHack(DOMNode *const nd)
  {
    if(!nd){
      return 1;
    }
    //if nodetype is textnode, then check for whitespace
    if(nd->getNodeType() == DOMNode::TEXT_NODE){
      const XMLCh* val = nd->getNodeValue();
      boost::scoped_array<char> cstrValue(XMLString::transcode(val));
      std::string sval = cstrValue.get();

      unsigned int  pos = sval.find_first_not_of("\n\t ");
      if(pos == std::string::npos){
	// ok we have a whitespace.. killing
	DOMNode *parent = nd->getParentNode();
	parent->removeChild(nd);
	return 2;
      }
    }else{
      //this is not a textnode and therefore we need to check it children if any
      if(nd->hasChildNodes()){
	DOMNodeList *children = nd->getChildNodes();
	for(unsigned int idx=0; idx<children->getLength(); idx++){
	  int ret = XercesHack(children->item(idx));
	  if(ret == 2){//ok we killed one
	    idx--;
	    children = nd->getChildNodes();
	  }
	}
      }
    }
    return 1;
  }

};
