/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "framework/stringutil.hpp"
#include "framework/dom_ptr.hpp"
#include "oapu/oapr_uuid.hpp"
//boost includes
#include "boost/smart_ptr.hpp"

//apache includes
#include <xercesc/dom/DOMWriter.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/framework/XMLFormatter.hpp>
#include <xercesc/framework/MemBufFormatTarget.hpp>
#include "framework/xmlutil.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oa
{

  DOMDocument*const  XMLUtils::Document(const std::string& uri,
					const std::string& root,
					bool unique){

    return XMLDocShare::GetDoc(uri,root,unique);
  }
  DOMDocument*const  XMLUtils::Document(const XMLCh *const uri,
					const XMLCh *const root,
					bool unique){
    return XMLDocShare::GetDoc(uri,root,unique);
  }
  int XMLUtils::Print(std::string& target, DOMNode *const toWrite)
  {
    if(!toWrite){
      return 0;
    }
    XMLCh tempStr[100];
    XMLString::transcode("LS",tempStr,99);
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(tempStr);
    dom_ptr<DOMWriter> theSerializer;
    boost::scoped_ptr<MemBufFormatTarget> formatTarget(new MemBufFormatTarget);
    theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();

    theSerializer->writeNode(formatTarget.get(),*toWrite);
    target = (char*)formatTarget->getRawBuffer();
    return formatTarget->getLen();
    
  }
  std::string XMLUtils::Print(DOMNode*const toWrite)
  {
    std::string target;
    if(!toWrite){
      return target;
    }
    XMLCh tempStr[100];
    XMLString::transcode("LS",tempStr,99);
    DOMImplementation *impl = DOMImplementationRegistry::getDOMImplementation(tempStr);
    dom_ptr<DOMWriter> theSerializer;
    boost::scoped_ptr<MemBufFormatTarget> formatTarget(new MemBufFormatTarget);
    theSerializer = ((DOMImplementationLS*)impl)->createDOMWriter();

    theSerializer->writeNode(formatTarget.get(),*toWrite);
    target = (char*)formatTarget->getRawBuffer();
    return target;

  }
  void XMLUtils::DomstrToStd(std::string& target, const XMLCh *const toWrite)
  {
    target.clear();
    if(toWrite){
      boost::scoped_ptr<char> tmp(XMLString::transcode(toWrite));
      target=tmp.get();
    }
  }
  void XMLUtils::DomstrToStd(std::wstring& target, const XMLCh*const toWrite)
  {
    target.clear();
    if(toWrite){
      target = (wchar_t*)toWrite;
    }
  }
  XMLCh* XMLUtils::StdToDomstr(const std::string &toWrite)
  {
    return XMLString::transcode(toWrite.c_str());
  }
  XMLCh* XMLUtils::StdToDomstr(const std::wstring &toWrite)
  {
    /**\todo how to handle this conversion?? is this even used anywhere?*/
    return 0;
  }
  int XMLUtils::Uuid(std::wstring &id)
  {
    /*
    oa::UuidGen*const ug=oa::UuidGen::Instance();
    std::string suid=ug->GetNext();
    */
    oapr::uuid ud;
    ud.regen();
    oa::StringUtil().stow(ud.str(),id);
    return id.length();
  }
  int XMLUtils::Uuid(std::string &id)
  {
    oapr::uuid ud;
    ud.regen();
    id=ud.str();
    return id.length();
  }
  bool XMLUtils::HasElem(DOMNode *const nd,const XMLCh*const name)
  {
    //first we check the node itself
    if(nd && nd->getNodeType() == DOMNode::ELEMENT_NODE){
      DOMNodeList *ndList = ((DOMElement*)nd)->getElementsByTagName(name);
      if(ndList && ndList->getLength()>0){
	return true;
      }
    }
    return false;
  }
  int XMLUtils::Clean(DOMNode *const nd)
  {
    if(!nd){
      return 1;
    }
    //if nodetype is textnode, then check for whitespace
    if(nd->getNodeType() == DOMNode::TEXT_NODE){
      const XMLCh* val = nd->getNodeValue();
      if(XMLString::isAllWhiteSpace(val)){
	// ok we have a whitespace.. killing
	DOMNode *parent = nd->getParentNode();
	parent->removeChild(nd);
	return 2;
      }
    }else{
      //this is not a textnode and therefore we need to check it children if any
      if(nd->hasChildNodes()){
	DOMNodeList* children = nd->getChildNodes();
	for(unsigned int idx=0; idx<children->getLength(); idx++){
	  DOMNode *childItem = children->item(idx);
	  int ret = Clean(childItem);
	  if(ret == 2){//ok we killed one
	    idx--;
	    children = nd->getChildNodes();
	  }
	}
      }
    }
    return 1;
  }
  bool XMLUtils::NeedsEscape(const std::string &str)
  {
    std::string::size_type fnd = str.find_first_of(gNeedsEscape);
    if(fnd == std::string::npos){
      return false;
    }else{
      return true;
    }
  }
  bool XMLUtils::NeedsEscape(const XMLCh *str)
  {
    if(XMLString::findAny(str,gxNeedsEscape)){
      return true;
    }else{
      return false;
    }
  }
  const XMLCh* XMLUtils::DecodeEscaped(const XMLCh*const str)
  {
    /* list of things to look for:
     * &lt; &gt; &amp; &apos; &qt;
     */
    XMLCh* orig = XMLString::replicate(str);
    unsigned int length = XMLString::stringLen(orig);
    unsigned int moveCount = 0;
    for(unsigned int idx=0; idx<length; ++idx){
      //first sign of escaped
      moveCount=0;
      if(orig[idx] == chAmpersand){
	//is it &lt;
	if(idx < length-4 &&
	   orig[idx+1]==chLatin_l &&
	   orig[idx+2]==chLatin_t &&
	   orig[idx+3]==chSemiColon){
	  //replace it
	  orig[idx]=chOpenAngle;
	  moveCount=4;

	}else if(idx < length-4 &&
		 orig[idx+1]==chLatin_g &&
		 orig[idx+2]==chLatin_t &&
		 orig[idx+3]==chSemiColon){
	  //its &gt;
	  orig[idx]=chCloseAngle;
	  moveCount=4;
	}else if(idx < length-5 &&
		 orig[idx+1]==chLatin_a &&
		 orig[idx+2]==chLatin_m &&
		 orig[idx+3]==chLatin_p &&
		 orig[idx+4]==chSemiColon){
	  //its &amp;
	  orig[idx]=chAmpersand;
	  moveCount=5;
	}else if(idx < length-6 &&
		 orig[idx+1]==chLatin_a &&
		 orig[idx+2]==chLatin_p &&
		 orig[idx+3]==chLatin_o &&
		 orig[idx+4]==chLatin_s &&
		 orig[idx+5]==chSemiColon){
	  //its &apos;
	  orig[idx]=chSingleQuote;
	  moveCount=6;
	}
	if(moveCount>0){
	  XMLString::moveChars(&orig[idx+1],&orig[idx+moveCount],(length-idx-1));
	  for(unsigned int midx=moveCount;midx>0; --midx){
	    orig[length-midx]=chNull;
	  }
	  //recalc length
	  length = XMLString::stringLen(orig);
	}	
      }
    }
    return orig;
  }
  bool XMLUtils::HasDecoded(const XMLCh*const str)
  {
    /* list of character sequences to find:
     * - gEscapedOpenAngle
     * - gEscapedCloseAngle
     * - gEscapedAmpersand
     * - gEscapedSingleQuote
     * - gEscapedDoubleQuote
     */
    if(Find(str,gEscapedOpenAngle)){
      return true;
    }else if(Find(str,gEscapedCloseAngle)){
      return true;
    }else if(Find(str,gEscapedAmpersand)){
      return true;
    }else if(Find(str,gEscapedSingleQuote)){
      return true;
    }else if(Find(str,gEscapedDoubleQuote)){
      return true;
    }
    return false;
  }
  const XMLCh*const XMLUtils::Find(const XMLCh*const str, 
				   const XMLCh*const sub)
  {
    unsigned int strLength=0,subLength=0;

    strLength = XMLString::stringLen(str);
    subLength = XMLString::stringLen(sub);
    unsigned int idx=0,subIdx=0;
    for(idx=0; idx<strLength; ++idx){
      subIdx=0;
      if(str[idx] == sub[0]){
	for(subIdx=1;subIdx<subLength;++subIdx){
	  if(idx+subIdx<strLength){
	    if(str[idx+subIdx]!=sub[subIdx]){
	      break;
	    }else if(str[idx+subIdx] == sub[subIdx] &&
	       subIdx==subLength-1){
	      return &str[idx];
	    }
	  }else{
	    break;
	  }
	}
      }
    }
    return 0;
  }



  XMLDocShare XMLDocShare::m_this;

  XMLDocShare& XMLDocShare::Instance(){
    return XMLDocShare::m_this;
  }

  void XMLDocShare::ReleaseAll()
  {
    XMLDocShare ds=XMLDocShare::Instance();
    cont_type::iterator pos=ds.m_documents.begin();
    for(;pos!=ds.m_documents.end(); ++pos){
      if(pos->second){
	pos->second->release();
	pos->second=0;
      }
    }
  }
  DOMDocument*const XMLDocShare::GetDoc(const std::string &uri,
					const std::string &root,
					bool unique)
  {
    //build a combo or uri + root to search for
    std::string key=uri+root;
    if(unique){
      std::string id;
      oa::XMLUtils().Uuid(id);
      key+=id;
    }
    cont_type::iterator find=XMLDocShare::m_this.m_documents.find(key);
    if(find == XMLDocShare::m_this.m_documents.end() ||
       !find->second){
      //create the document
      
      XMLCh tempStr[100];
      XMLString::transcode("LS",tempStr, 99);
      DOMImplementation *imp=DOMImplementationRegistry::getDOMImplementation(tempStr);
      boost::scoped_array<XMLCh> xuri;
      boost::scoped_array<XMLCh> xpre;
      xuri.reset(XMLString::transcode(uri.c_str()));
      xpre.reset(XMLString::transcode(root.c_str()));
      DOMDocument* theDoc=imp->createDocument(xuri.get(),xpre.get(),0);
      XMLDocShare::m_this.m_documents[key]=theDoc;
      return theDoc;
    }else{
      //simply return the doc
      return find->second;
    }
  }
  DOMDocument*const XMLDocShare::GetDoc(const XMLCh* const uri,
					const XMLCh* const root,
					bool unique)
  {
    /**\todo implement a native XMLCh* */
    boost::scoped_array<char> suri,sroot;
    suri.reset(XMLString::transcode(uri));
    sroot.reset(XMLString::transcode(root));
    return XMLDocShare::GetDoc(std::string(suri.get()),
			       std::string(sroot.get()),
			       unique);
  }
};
