/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef FRAMEWORK_XMLUTIL_H
#define FRAMEWORK_XMLUTIL_H
#include "framework/framework_exports.h"
//std includes
#include <string>
#include <map>
#include <iostream>
//apache includes
#include <xercesc/dom/DOMDocument.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XMLUniDefs.hpp>

static const char *const gNeedsEscape = "<>&\'\"";
static const XMLCh gxNeedsEscape[] =
{
  xercesc::chOpenAngle, xercesc::chCloseAngle, xercesc::chAmpersand, 
  xercesc::chSingleQuote, xercesc::chDoubleQuote, xercesc::chNull
};
static const XMLCh gEscapedOpenAngle[] = 
{
  xercesc::chAmpersand,
  xercesc::chLatin_l,
  xercesc::chLatin_t,
  xercesc::chSemiColon,
  xercesc::chNull
};
static const XMLCh gEscapedCloseAngle[] =
{
  xercesc::chAmpersand,
  xercesc::chLatin_g,
  xercesc::chLatin_t,
  xercesc::chSemiColon,
  xercesc::chNull
};

static const XMLCh gEscapedAmpersand[] = 
{
  xercesc::chAmpersand,
  xercesc::chLatin_a,
  xercesc::chLatin_m,
  xercesc::chLatin_p,
  xercesc::chSemiColon,
  xercesc::chNull
};

static const XMLCh gEscapedSingleQuote[] =
{
  xercesc::chAmpersand,
  xercesc::chLatin_a,
  xercesc::chLatin_p,
  xercesc::chLatin_o,
  xercesc::chLatin_s,
  xercesc::chSemiColon,
  xercesc::chNull
};
static const XMLCh gEscapedDoubleQuote[] = 
{
  xercesc::chAmpersand,
  xercesc::chLatin_q,
  xercesc::chLatin_u,
  xercesc::chLatin_o,
  xercesc::chLatin_t,
  xercesc::chSemiColon,
  xercesc::chNull
};
class XMLFormatter;

namespace oa
{
#ifdef WIN32
    class FW_API XMLUtils;
    class FW_API XMLInit;
#endif
  /**
   * Class to provide a collection of misc xml utility functions
   * for using XercesC in STL environments.
   * These are mainly for convenience
   */
  class XMLUtils
  {
  public:
    /**
     * Create a document 
     */
    xercesc::DOMDocument *const Document(const std::string& uri,
				const std::string& root,
				bool unique=false);
    xercesc::DOMDocument *const Document(const XMLCh* const uri,
				const XMLCh* const root,
				bool unique=false);
    //    static void ReleaseDocument();
    /**
     * Takes a DOM_Node and prints it to a string in typical
     * xml style formatting
     *
     * @param target the output string
     * @param toWrite DOM_Node to write
     * @return zero on success, non-zero otherwise
     */
    int Print(std::string& target, xercesc::DOMNode *const toWrite);
    std::string Print(xercesc::DOMNode*const toWrite);
    /**
     * Converts a DOMString to std::wstring
     * There really is no conversion process. However, DOMString
     * is not guaranteed to be null terminated and the buffer
     * may be shared with other strings.
     *
     * @param target The string to write to
     * @param toWrite The DOMString to convert
     * @return the length of the converted target
     */
    void DomstrToStd(std::wstring& target, const XMLCh*const toWrite);
    void DomstrToStd(std::string& target, const XMLCh*const toWrite);
    /**
     * Converts a std::wstring to a DOMString
     * There is no explicit conversion between types
     * the function is used to guarantee no side affects
     *
     * @param target the DOMString to convert to
     * @param toWrite the std::wstring to convert from
     * @return the length of the DOMString
     */
    XMLCh* StdToDomstr(const std::wstring &toWrite);
    XMLCh* StdToDomstr(const std::string &toWrite);
    /**
     * Generates a UID
     * A unique identifier 
     *
     */
    int Uuid(std::wstring &id);
    int Uuid(std::string &id);

    /**
     * Determine whether a node has an element
     * @param nd The node to search under
     * @param name The name to look for
     */
    bool HasElem(xercesc::DOMNode *const nd, const XMLCh*const name);
    /**
     * Removes all extraneous DOM_Text nodes which contain
     * nothing but whitespace. Usually as a result for
     * pretty xml.. e.g.
     * <foo>
     *   <bar/>
     * </foo>
     */
    int Clean(xercesc::DOMNode*const nd);
    /**
     * Checks for characters in a string which need to be escaped in xml
     * @param str String to check for escapable characters
     * @return non-zero if contains escapable chars, zero otherwise
     */
    bool NeedsEscape(const std::string &str);
    bool NeedsEscape(const XMLCh* str);

    /**
     *
     */
    const XMLCh* DecodeEscaped(const XMLCh*const str);
    bool HasDecoded(const XMLCh*const str);
    const XMLCh*const Find(const XMLCh*const str, 
			   const XMLCh*const sub);
  protected:

    /**
     * Performs initialization for the string
     * used mainly for locale converstion etc...
     */
    //    int Print(std::string& target, XMLCh* toWrite);
    /**
     * ummmm whatever
     */
    /*
    int Print(DOMNode*const toWrite,
		     XMLFormatter *ft);
    */


  };
  
  class XMLDocShare
  {
  public:
    static void ReleaseAll();
    static xercesc::DOMDocument*const GetDoc(const std::string &uri,
				    const std::string &root,
				    bool unique=false);
    static xercesc::DOMDocument*const GetDoc(const XMLCh *const uri,
				    const XMLCh *const root,
				    bool unique=false);

#ifndef WIN32
  private:	
#else
  public:
#endif
	  ~XMLDocShare(){}
  private:
    XMLDocShare(){}

    static XMLDocShare& Instance();

    typedef std::map<std::string,xercesc::DOMDocument*> cont_type;
    cont_type m_documents;

    static XMLDocShare m_this;
  };

};
#endif
