/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABAGENT_H
#define OAJABBER_IJABAGENT_H
#include <vector>
#include <boost/smart_ptr.hpp>
#include "jabber/ijabiq.h"

namespace jabber
{
  class OAJABBER_API JabAgent;
  class OAJABBER_API JabAgents;
  

  /** @ingroup gJabPacket
   */

  ///Jabber Agent packet
  /**
   * Packet containing info about an agent
   */
  class JabAgent : public JabIq
  {
  public:
    JabAgent()
      : JabIq("agent"){}
    JabAgent(const JabAgent &ja)
      : JabIq(ja),
      m_jid(ja.m_jid),
      m_name(ja.m_name),
      m_description(ja.m_description),
      m_transport(ja.m_transport),
      m_groupchat(ja.m_groupchat),
      m_service(ja.m_service),
      m_register(ja.m_register),
      m_search(ja.m_search){}
    virtual ~JabAgent(){}
    
    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:agent";
    }
    ///The xml namespace this is under
    virtual std::string Xmlns()const{
      return "jabber:iq:agent";
    }

    ///The jabber ID of the agent
    /** @param jid The jabber ID*/
    virtual void Jid(const std::string &jid){m_jid=jid;}

    ///The name of the agent
    /** @param name The name*/
    virtual void Name(const std::string &name){m_name=name;}

    ///A description of the agent
    /** @param desc the description*/
    virtual void Description(const std::string &desc){m_description=desc;}

    ///Whether the agent is a transport
    /** @param trans true if it is a transport, false otherwise*/
    virtual void Transport(bool trans){m_transport=trans;}

    ///Whether the agent supports groupchat
    /** @param grp true if it supprots groupchat,false otherwise*/
    virtual void GroupChat(bool grp){m_groupchat=grp;}

    ///The service the agent  provides
    /** @param srv the service*/
    virtual void Service(const std::string &srv){m_service=srv;}

    ///Whether a user has to register
    /** @param reg true if registration is requred, false otherwise*/
    virtual void Register(bool reg){m_register=reg;}

    ///Whether the agent is searchable
    /** @param srch true if searchable, false otherwise*/
    virtual void Search(bool srch){m_search=srch;}


    ///The jabber ID of the agent
    /** @return The jabber ID*/
    virtual std::string Jid()const{return m_jid;}

    ///The name of the agent
    /** @return The name*/
    virtual std::string Name()const{return m_name;}

    ///A description of the agent
    /** @return the description*/
    virtual std::string Description()const{return m_description;}

    ///Whether the agent is a transport
    /** @return true if it is a transport, false otherwise*/
    virtual bool Transport()const{return m_transport;}

    ///Whether the agent supports groupchat
    /** @return true if it supprots groupchat,false otherwise*/
    virtual bool GroupChat()const{return m_groupchat;}

    ///The service the agent  provides
    /** @return the service*/
    virtual std::string Service()const{return m_service;}

    ///Whether a user has to register
    /** @return true if registration is requred, false otherwise*/
    virtual bool Register()const{return m_register;}

    ///Whether the agent is searchable
    /** @return true if searchable, false otherwise*/
    virtual bool Search()const{return m_search;}


    ///Build a DOMNode from data
    /**
     * Constructs a DOMNode pointer from the data
     * This is used by xml() to construct the string
     * @return DOMNode pointer, null if failure
     *
     * @exceptions oaexc::LogicError
     */
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    virtual xercesc::DOMNode*const BuildProperties();
  protected:
    std::string m_jid;
    std::string m_name;
    std::string m_description;
    bool m_transport;
    bool m_groupchat;
    std::string m_service;
    bool m_register;
    bool m_search;    

    friend class JabAgents;
  };

  ///collection of JabAgent
  class JabAgents : public JabIq
  {
  public:
    JabAgents();
    JabAgents(const JabAgents &ja)
      : JabIq(ja),
	m_agents(ja.m_agents){}
    virtual ~JabAgents();

    virtual const char*const Uri()const{
      return "xmpp:xmlns=jabber:iq:agents";
    }
    ///Returns the xml namespace this works under
    virtual std::string Xmlns()const{
      return "jabber:iq:agents";
    }

    int Length();
    ///Retrieve Agent at index
    /**
     * Retrieve an pointer to agent at index i
     * @param i index to retrieve agent poitner
     * @exceptions std::range_error
     */
    JabAgent*const Item(int i);
    int Insert(JabAgent *agent);


    virtual xercesc::DOMNode*const BuildXml();
  protected:
    typedef boost::shared_ptr<JabAgent> agent_ptr;
    typedef std::vector<agent_ptr> vec_agents;
    vec_agents m_agents;
  };
};
#endif
