/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlstr.hpp"
#include "jabber/ijabiq.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  JabIq::JabIq(std::string classid) : JabPacket("iq",classid){}
  JabIq::~JabIq(){}
  std::string JabIq::Id()const{
    if(m_id.empty()){
      oa::XMLUtils().Uuid(m_id);
    }
    return m_id;
  }

  std::string JabIq::xml()
  {
    std::string ret;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }

  DOMNode* JabIq::BuildIq()
  {
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;
    JabJid to,from;
    std::string type,id;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    tagName.set("iq");
    oa::dom_ptr<DOMElement> iqElem(doc->createElement(tagName.get()));

    /* getting the type of the iq.
     * - this *must* be get if not specified
     */
    type=Type();
    if(type.empty()){
      type="get";//setting default to get
    }
    if(type=="error"){
      oa::dom_ptr<DOMNode> errnd(BuildError());
      iqElem->appendChild(errnd.release());
    }

    attrName.set("type");
    attrValue.set(type.c_str());
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* getting the to attribute
     * - this is *required* if empty throw a logic exception
     * - note: change to be not required, as per the docs
     */
    to=To();
    if(!to.IsNull()){
      attrName.set("to");
      attrValue.set(to.Jid().c_str());
      iqElem->setAttribute(attrName.get(),attrValue.get());
    }

    /* get the id attribute
     * - this is required, so throw a logic exception
     */
    id=Id();
    if(id.empty()){
      OAPR_THROW(oapr::logic_error,"Invalid ID attribute");
    }
    attrName.set("id");
    attrValue.set(id.c_str());
    iqElem->setAttribute(attrName.get(),attrValue.get());

    /* get the from
     * this is optional, since the jabberd takes care of
     * it if its not here
     */
    from=From();
    if(!from.IsNull()){
      attrName.set("from");
      attrValue.set(from.Jid().c_str());
      iqElem->setAttribute(attrName.get(),attrValue.get());
    }
    return iqElem.release();
  }
  DOMNode* JabIq::BuildError()
  {
    std::string code,msg;
    oa::xmlstr<XMLCh,char> tagName,attrName,attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    if(!HasError()){
      /* ok this is bad called this on a no error condition*/
      OAPR_THROW(oapr::logic_error,"Tried to build error, when there is no error");
    }
    JabError jerror = Error();
    tagName.set("error");
    oa::dom_ptr<DOMElement> errorElem(doc->createElement(tagName.get()));

    /* code attribute is required. this is a bad situation and
     * no reasonable way to recover
     */
    code=jerror.Code();
    if(code.empty()){
      OAPR_THROW(oapr::logic_error,"Tried to build error, no code value set");
    }

    attrName.set("code");
    attrValue.set(code.c_str());
    errorElem->setAttribute(attrName.get(),attrValue.get());


    /* message is NOT required by the standard.
     * Appendix A of the xmpp core doc has this to say:
     *
     * "A standard error element is used for failed processing
     *  of XML chunks. This element is a child of the failed
     *  element and MUST include a 'code' attribute corresponding 
     * to one of the following error codes. "
     *
     * seeing as how there is no mention of the textnode in regards to
     * the <error/> I must conclude that it is not required.
     */
    msg=jerror.Message();
    if(!msg.empty()){
      /*note: using tagName for simplicity sake, so I
       * don't have to setup tagValue ...
       */
      tagName.set(msg.c_str());
      oa::dom_ptr<DOMText> msgValue(doc->createTextNode(tagName.get()));
      errorElem->appendChild(msgValue.release());
    }
    /* the caller is the owner, which is why the return is non-const
     * so we have to release... this I do NOT like... but oh well
     */
    return errorElem.release();
  }
};
