/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMText.hpp>
#include <xercesc/dom/DOMAttr.hpp>
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "jabber/ijabmsg.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  JabMsg* JabMsg::Reply()
  {
    return 0;
  }

  std::string JabMsg::Id()const
  {
    if(m_id.empty()){
      oa::XMLUtils().Uuid(m_id);
    }
    return m_id;
  }
  void JabMsg::Extension(const x_ptr& xp)
  {
    if(xp){
      x_ptr clone(xp->Clone());
      clone->Attached(true);//set the attached to true
      m_extensions.push_back(clone);
    }
  }
  std::string JabMsg::xml()
  {
    std::string ret;
    oa::dom_ptr<DOMNode> built(BuildXml());

    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }
  DOMNode*const JabMsg::BuildXml()
  {
    JabJid to,from;
    std::string id,type,body,subject,thread;
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    tagName.set("message");
    DOMElement *msgElem = doc->createElement(tagName.get());

    to=To();
    if(!to.IsNull()){
      attrName.set("to");
      attrValue.set(to.Jid().c_str());
      DOMAttr *toAttr = doc->createAttribute(attrName.get());
      toAttr->setValue(attrValue.get());
      msgElem->setAttributeNode(toAttr);
    }

    from=From();
    if(!from.IsNull()){
      attrName.set("from");
      attrValue.set(from.Jid().c_str());
      DOMAttr *fromAttr = doc->createAttribute(attrName.get());
      fromAttr->setValue(attrValue.get());
      msgElem->setAttributeNode(fromAttr);
    }

    id=Id();
    if(!id.empty()){
      attrName.set("id");
      attrValue.set(id.c_str());
      DOMAttr *idAttr = doc->createAttribute(attrName.get());
      idAttr->setValue(attrValue.get());
      msgElem->setAttributeNode(idAttr);
    }

    /* NOTE: we *have* to have a type, it defaults to normal */
    type=Type();
    if(type.empty()){      
      type="normal";
    }

    /* if we have an error, force the type to error */
    if(HasError()){
      type="error";
      attrName.set("type");
      attrValue.set(type.c_str());
      DOMAttr *typeAttr=doc->createAttribute(attrName.get());
      typeAttr->setValue(attrValue.get());//<-- bug#108
      
      JabError jerror=Error();
      tagName.set("error");
      tagValue.set(jerror.Message().c_str());
      DOMElement *errorElem=doc->createElement(tagName.get());
      DOMText *errorValue=doc->createTextNode(tagValue.get());
      errorElem->appendChild(errorValue);

      attrName.set("code");
      attrValue.set(jerror.Code().c_str());
      DOMAttr *codeAttr = doc->createAttribute(attrName.get());
      codeAttr->setValue(attrValue.get());
      errorElem->setAttributeNode(codeAttr);
      msgElem->appendChild(errorElem);
      msgElem->setAttributeNode(typeAttr);
    }else{
      attrName.set("type");
      attrValue.set(type.c_str());
      DOMAttr *typeAttr = doc->createAttribute(attrName.get());
      typeAttr->setValue(attrValue.get());
      msgElem->setAttributeNode(typeAttr);
    }


    body=Body();
    if(!body.empty()){
      tagName.set("body");
      tagValue.set(body.c_str());
      DOMElement *bodyElem=doc->createElement(tagName.get());
      DOMText *bodyValue=doc->createTextNode(tagValue.get());
      bodyElem->appendChild(bodyValue);
      msgElem->appendChild(bodyElem);
    }

    subject=Subject();
    if(!subject.empty()){
      tagName.set("subject");
      tagValue.set(subject.c_str());
      DOMElement *subjectElem=doc->createElement(tagName.get());
      DOMText *subjectValue=doc->createTextNode(tagValue.get());
      subjectElem->appendChild(subjectValue);
      msgElem->appendChild(subjectElem);
    }

    thread=Thread();
    if(!thread.empty()){
      tagName.set("thread");
      tagValue.set(thread.c_str());
      DOMElement *threadElem = doc->createElement(tagName.get());
      DOMText *threadValue = doc->createTextNode(tagValue.get());
      threadElem->appendChild(threadValue);
      msgElem->appendChild(threadElem);
    }

    /* build extensions if we have any*/
    BuildExtension(msgElem);
    return msgElem;
  }
  void JabMsg::BuildExtension(DOMNode*const msg)
  {
    if(!msg){
      /**\todo throw like a moho*/
      return;
    }
    DOMNode* xNode=0;
    JabMsg::x_cont::iterator pos=m_extensions.begin();
    for(;pos!=m_extensions.end();++pos){
      xNode = (*pos)->BuildXml();
      if(xNode){//<-- super paranoia this is
	msg->appendChild(xNode);
      }
    }
  }
};
