/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IJABMSG_H
#define OAJABBER_IJABMSG_H
#include <list>
#include "jabber/oajabber_exports.h"
#include "xstrm/jabpacket.h"
#include "xstrm/ijabx.h"

namespace jabber
{
#ifdef WIN32
  class OAJABBER_API JabMsg;
#endif
  /** @ingroup gJabPacket
   */
  /**
   * JabMsg is the primary object created from a
   * <message> node being received from jabber.
   * Any other specialization of JabMsg, e.g.
   * an X element is not handled.

   * Set the type of the message
   * Allowed values are:
   * - normal   Single message
   * - chat     Traditional two-way chat
   * - groupchat Chat among multiple entities
   * - headline Ticker or active list of items
   * - error    See the error element
   *
   * Refer to the official docs for what is acceptable
   * @param tp The value of type
   
   */
  class JabMsg : public JabPacket
  {
  public:
    typedef JabX x_type;
    typedef boost::shared_ptr<x_type> x_ptr;
    typedef std::list<x_ptr> x_cont;
  public:
    JabMsg()
      : JabPacket("message","message"){}
    JabMsg(const JabMsg &jm)
      : JabPacket(jm), 
        m_subject(jm.m_subject),
        m_body(jm.m_body),
        m_thread(jm.m_thread)
    {
      /* note: can't use stl algos because of boost pointer*/
      x_cont::const_iterator pos=jm.m_extensions.begin();
      for(;pos!=jm.m_extensions.end();++pos){
	m_extensions.push_back((*pos)->Clone());
      }
    }
    virtual ~JabMsg(){}
    
    virtual const char*const Uri()const{
      return "xmpp:message";
    }
    JabMsg* Reply();

    using JabPacket::Id;
    virtual std::string Id()const;
    /**
     * Set the body of the message 
     * @param bdy The value to set
     */
    virtual void Body(const std::string &bdy){
      m_body=bdy;
    }
    /**
     * Set the subject of the message
     * @param sub The value to set
     */
    virtual void Subject(const std::string &sub){
      m_subject=sub;
    }
    /**
     * Set the thread of the message
     * @param thr The value to set
     */
    virtual void Thread(const std::string &thr){
      m_thread=thr;
    }
    /**
     * Get the value of the body
     * @return returns the value of the body
     */
    virtual std::string Body()const{
      return m_body;
    }
    /**
     * Get the value of subject
     * @return the value of the subject, may be empty
     */
    virtual std::string Subject()const{
      return m_subject;
    }
    /**
     * Get the value of thread
     * @return the value of the thread, may be empty
     */
    virtual std::string Thread()const{
      return m_thread;
    }

    /**
     * Add an extension to this message
     */
    virtual void Extension(const x_ptr& xp);
    /**
     * Get the generated xml. Will create an 
     * appropriate xml chunk to stream to a jabber server
     * @return xml
     */
    virtual std::string xml();
    virtual xercesc::DOMNode*const BuildXml();
  protected:
    /**
     * Builds the extension dom and appends each to the
     * msg param
     */
    virtual void BuildExtension(xercesc::DOMNode*const msg);
  protected:
    std::string m_subject;          //subject of msg
    std::string m_body;             //body of msg
    std::string m_thread;           //thread for msg (replies only)
    std::list<x_ptr> m_extensions;
  };
};
#endif
