/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "boost/smart_ptr.hpp"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldagent.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  /***********************************************************
   * BuildAgent
   ***********************************************************/
  bool BuildAgent::CanBuild(DOMNode *const nd)const
  {
    /* This should never qualify, since its used
     * strictly for BuildAgents convenience
     */
    return false;
  }
  JabPacket* BuildAgent::Build(DOMNode *const nd)
  {
    std::auto_ptr<JabAgent> pagent(new JabAgent);
    if(!nd || !nd->hasChildNodes()){return 0;}


    DOMNode* agent = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Agent",oapr::Log_Info);

    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }
#endif
    const XMLCh* agentName = ((DOMElement*)agent)->getTagName();
    //no matter what this is... check for errors...
    JabError error;
    if(GetError(error,agent)){
      pagent->Error(error);
    }

    std::string strAgentName;
    oa::XMLUtils().DomstrToStd(strAgentName,agentName);
    if(strAgentName == "iq"){
      bool trans(false),gc(false),reg(false),search(false);
      std::string to,from,type,id,jid,name,desc,service;

      GetTo(to,agent);
      GetFrom(from,agent);
      GetType(type,agent);
      GetId(id,agent);
      GetJid(jid,agent);
      GetName(name,agent);
      GetDescription(desc,agent);
      GetTransport(trans,agent);
      GetGroupChat(gc,agent);
      GetService(service,agent);
      GetRegister(reg,agent);
      GetSearch(search,agent);

      pagent->To(to);
      pagent->From(from);
      pagent->Type(type);
      pagent->Id(id);
      pagent->Jid(jid);
      pagent->Name(name);
      pagent->Description(desc);
      pagent->Transport(trans);
      pagent->GroupChat(gc);
      pagent->Service(service);
      pagent->Register(reg);
      pagent->Search(search);
    }else{
      bool trans(false),gc(false),reg(false),search(false);
      std::string jid,name,desc,service;
      
      GetJid(jid,nd);
      GetName(name,nd);
      GetDescription(desc,nd);
      GetTransport(trans,nd);
      GetGroupChat(gc,nd);
      GetService(service,nd);
      GetRegister(reg,nd);
      GetSearch(search,nd);

      pagent->Jid(jid);
      pagent->Name(name);
      pagent->Description(desc);
      pagent->Transport(trans);
      pagent->GroupChat(gc);
      pagent->Service(service);
      pagent->Register(reg);
      pagent->Search(search);
    }

    return pagent.release();
  }

  bool BuildAgent::GetJid(std::string &jid,DOMNode*const node)
  {
    if(!node){
      return false;
    }

    boost::scoped_array<XMLCh> attrName(XMLString::transcode("jid"));
    const XMLCh* domJid = ((DOMElement*)node)->getAttribute(attrName.get());
    if(!domJid){
      return false;
    }
    std::string strJid;
    oa::XMLUtils().DomstrToStd(jid,domJid);
    return true;
  }
  bool BuildAgent::GetName(std::string& name,DOMNode*const node)
  {
    if(!node){
      return false;
    }

    boost::scoped_array<XMLCh> tagName(XMLString::transcode("name"));
    DOMNodeList* names = ((DOMElement*)node)->getElementsByTagName(tagName.get());
    if(names->getLength()<=0){
      return false;
    }
    DOMNode *ndName = names->item(0);
    if(!ndName){
      return false;
    }

    DOMNode* txt = ndName->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domName = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(name,domName);
    return true;
  }
  bool BuildAgent::GetDescription(std::string&desc,DOMNode*const node)
  {
    if(!node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("description"));
    DOMNodeList* descs = ((DOMElement*)node)->getElementsByTagName(tagName.get());
    if(descs->getLength()<=0){
      return false;
    }
    DOMNode *ndDesc = descs->item(0);
    if(!ndDesc){
      return false;
    }
    DOMNode* txt = ndDesc->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domDesc = txt->getNodeValue();
    std::string strDesc;
    oa::XMLUtils().DomstrToStd(desc,domDesc);
    return true;
  }
  bool BuildAgent::GetTransport(bool &trans,DOMNode*const node)
  {
    if(!node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("transport"));
    if(oa::XMLUtils().HasElem(node,tagName.get())){
      trans=true;
    }else{
      trans=false;
    }
    return true;
  }
  bool BuildAgent::GetGroupChat(bool &gc,DOMNode*const node)
  {
    if(node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("groupchat"));
    if(oa::XMLUtils().HasElem(node,tagName.get())){
      gc = true;
    }else{
      gc = false;
    }
    return true;
  }
  bool BuildAgent::GetService(std::string& serv,DOMNode*const node)
  {
    if(!node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("service"));
    DOMNodeList* srvs = ((DOMElement*)node)->getElementsByTagName(tagName.get());
    if(srvs->getLength()<=0){
      return false;
    }
    DOMNode *srv = srvs->item(0);
    if(!srv){
      return false;
    }
    DOMNode* txt = srv->getFirstChild();
    if(!txt){
      return false;
    }
    const XMLCh* domServ = txt->getNodeValue();
    std::string strServ;
    oa::XMLUtils().DomstrToStd(serv,domServ);
    return true;
  }
  bool BuildAgent::GetRegister(bool &reg,DOMNode*const node)
  {
    if(!node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("register"));
    if(oa::XMLUtils().HasElem(node,tagName.get())){
      reg=true;
    }else{
      reg=false;
    }
    return true;
  }
  bool BuildAgent::GetSearch(bool &search,DOMNode*const node)
  {
    if(!node){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("search"));
    if(oa::XMLUtils().HasElem(node,tagName.get())){
      search = true;
    }else{
      search =false;
    }
    return true;
  }
  /***********************************************************
   * BuildAgents
   ***********************************************************/
  bool BuildAgents::CanBuild(DOMNode *const nd)const
  {
    /* To qualify it must:
     * - nd must be named iq
     * - the child must be query in jabber:iq:agents namespace
     */
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> queryName(XMLString::transcode("query"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("jabber:iq:agents"));
    if(nd->hasChildNodes() &&
       !XMLString::compareString(iqName.get(),nd->getNodeName())){
      DOMNode*child=nd->getFirstChild();
      if(!XMLString::compareString(queryName.get(),child->getNodeName()) &&
	 !XMLString::compareString(uri.get(),child->getNamespaceURI())){
	return true;
      }
    }    
    return false;
  }
  JabPacket* BuildAgents::Build(DOMNode *const nd)
  {
    if(!nd){return 0;}
    std::auto_ptr<JabAgents> pagents(new JabAgents);
    DOMNode *agents = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Agents",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }
#endif
    std::string to,from,type,id;
    GetTo(to,agents);
    GetFrom(from,agents);
    GetType(type,agents);
    GetId(id,agents);

    pagents->To(to);
    pagents->From(from);
    pagents->Type(type);
    pagents->Id(id);

    GetAgents(pagents.get(),agents);
    return pagents.release();
  }

  int BuildAgents::GetAgents(JabAgents*const agents,DOMNode*const nd)
  {
    std::auto_ptr<JabAgent> pagent;
    boost::scoped_ptr<BuildAgent> bagent;

    boost::scoped_array<XMLCh> tagName(XMLString::transcode("agent"));
    DOMNodeList* domAgents = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    for(unsigned int i=0;i<domAgents->getLength();++i){
      bagent.reset(new BuildAgent);
      pagent.reset(dynamic_cast<JabAgent*>(bagent->Build(domAgents->item(i))));
      if(!pagent.get()){
	continue;
      }
      agents->Insert(pagent.release());
    }
    return 0;
  }
};
