/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABBLDAGENT_H
#define OAJABBER_JABBLDAGENT_H
#include "xstrm/jabbuild.h"
#include "jabber/ijabagent.h"


namespace jabber
{
  class OAJABBER_API BuildAgent;
  class OAJABBER_API BuildAgents;

  /** @ingroup gJabBuild
   */

  ///Build an agent packet 
  /**
   * Builds a JabAgent packet from
   * a DOMNode. Providing that the DOMNode
   * contains the necessary xml
   */
  class BuildAgent : public JabBuild
  {
  public:

    ///ctor
    /**
     * Construct a builder with the DOMNode
     */
    BuildAgent()
      : JabBuild(){}
    BuildAgent(const BuildAgent& ba)
      : JabBuild(ba){}

    virtual ~BuildAgent(){}

    virtual std::string ID()const{
      return "jabber::BuildAgent";
    }
    virtual JabBuild* Clone(){
      return new BuildAgent(*this);
    }
    
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    /// Build the packet
    /**
     * Main public function to execute the build process
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  protected:
    ///Get the jid attribute on node
    /**
     * Get the value of the jid attribute on 
     * the node, and assign it to the member variable
     * @param jid the value of the jid
     * @param node The xercesc::DOMNode containing a jid attribute
     * @return false if failed, true if successful
     */
    virtual bool GetJid(std::string &jid,xercesc::DOMNode*const node);

    ///Get the name attribute on node
    /**
     * Get the value of the name attribute on node
     * @param name value of name
     * @param node The DOMNode containing a name attribute
     * @return false if failed, true if sucessful
     */
    virtual bool GetName(std::string &name,xercesc::DOMNode*const node);

    ///Get the value of description element
    /**
     * If there is a description element
     * get the value of the text child node
     * @param desc description string, i.e. output
     * @param node DOMNode containing a <description>
     * @return true if successful, false otherwise
     */
    virtual bool GetDescription(std::string &desc,xercesc::DOMNode*const node);

    ///Get the transport flag
    /**
     * Find and set the transport flag
     * @param trans transport value
     * @param node DOMNode containing a <transport/>
     * @return true if successful, false otherwise
     */
    virtual bool GetTransport(bool &trans,xercesc::DOMNode*const node);

    ///Get the groupchat flag
    /**
     * Find and set the groupchat flag
     * @param gc the groupchat flag, i.e. output
     * @param node DOMNode containing a <groupchat/>
     * @return true if successful, false otherwise
     */
    virtual bool GetGroupChat(bool &gc,xercesc::DOMNode*const node);

    ///Get the service name
    /**
     * Find the service name if it exists
     * @param serv serv string, i.e. output
     * @param node DOMNode containing a <service>
     * @return true if successful, false otherwise
     */
    virtual bool GetService(std::string &serv,xercesc::DOMNode*const node);

    ///Get the register flag
    /**
     * Find the register flag
     * @param reg register value
     * @param node DOMNode containing a <register/>
     * @return true if successful, false otherwise
     */
    virtual bool GetRegister(bool &reg,xercesc::DOMNode*const node);

    ///Get the searchable flag
    /**
     * Find the searchable flag
     * @param search search value
     * @param node DOMNode containing a <search/>
     * @return true if successful, false otherwise
     */
    virtual bool GetSearch(bool &search,xercesc::DOMNode*const node);
  };

  ///List of JabAgent 
  /**
   * THis is simply a list of agents
   * Primary diff from JabAgent is that each agent has a jid
   * and the query is in a different namespace
   */
  class BuildAgents : public JabBuild
  {
  public:
    ///ctor
    BuildAgents()
      : JabBuild(){}
    BuildAgents(const BuildAgents& ba)
      : JabBuild(ba){}

    ///dtor
    virtual ~BuildAgents(){}

    virtual std::string ID()const{
      return "jabber::BuildAgents";
    }
    virtual JabBuild* Clone(){
      return new BuildAgents(*this);
    }
    virtual bool CanBuild(xercesc::DOMNode *const nd)const;
    ///Build the packet
    /**
     * Build the packet
     * @param nd node to build a packet for
     * @return A jabpacket on success, null on failure
     */
    virtual JabPacket* Build(xercesc::DOMNode *const nd);
  protected:

    ///Build the agents list
    /**
     * Build the agents, this utilizes the BuildAgent
     * @param agents list of agents
     * @param nd DOMNode containing list of <agent>
     * @return zero on success
     */
    virtual int GetAgents(JabAgents*const agents,xercesc::DOMNode*const nd);
  };
};
#endif
