/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "boost/smart_ptr.hpp"
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldroster.h"
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  bool BuildRoster::CanBuild(DOMNode *const nd)const
  {
    /* requiements
     * - nd name is iq and it has children
     * - child is query with xmlns of jabber:iq:roster
     */
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> queryName(XMLString::transcode("query"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("jabber:iq:roster"));
    if(nd->hasChildNodes() &&
       !XMLString::compareString(nd->getNodeName(),iqName.get())){
      DOMNode*query=nd->getFirstChild();
      if(!XMLString::compareString(queryName.get(),query->getNodeName()) &&
	 !XMLString::compareString(uri.get(),query->getNamespaceURI())){
	return true;
      }
    }
    return false;
  }
  JabPacket* BuildRoster::Build(DOMNode *const nd)
  {
    std::auto_ptr<JabRoster> proster(new JabRoster);

    if(!nd){return 0;}
    if(!nd->hasChildNodes()){return 0;}
    DOMNode* iq = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Roster",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }
#endif
    JabError error;
    if(GetError(error,iq)){
      proster->Error(error);
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("item"));
    std::auto_ptr<JabRosterItem> ritem;
    DOMNodeList *items=((DOMElement*)iq)->getElementsByTagName(tagName.get());
    for(unsigned int i=0; i<items->getLength(); ++i){
      ritem.reset(new JabRosterItem);
      DOMNode* item = items->item(i);
      GetItem(ritem.get(),item);
      proster->Add(ritem.release());
    }
    return proster.release();
  }
  bool BuildRoster::GetItem(JabRosterItem*const item,DOMNode*const nd)
  {
    if(!nd){
      return false;
    }

    std::string name,jid,sub,group;
    GetName(name,nd);
    GetJid(jid,nd);
    GetSub(sub,nd);
    GetGroup(group,nd);

    item->Name(name);
    item->Jid(jid);
    item->SubType(sub);
    item->Group(group);
    return true;
  }
  bool BuildRoster::GetName(std::string &name,DOMNode*const nd)

  {
    if(!nd){
      return false;
    }
    
    boost::scoped_array<XMLCh> attrName(XMLString::transcode("name"));
    const XMLCh* domName=((DOMElement*)nd)->getAttribute(attrName.get());
    if(!domName){
      return false;
    }
    oa::XMLUtils().DomstrToStd(name,domName);
    return true;
  }
  bool BuildRoster::GetJid(std::string& jid,DOMNode*const nd)
  {
    if(!nd){
      return false;
    }
    boost::scoped_array<XMLCh> attrName(XMLString::transcode("jid"));
    const XMLCh* domJid = ((DOMElement*)nd)->getAttribute(attrName.get());
    if(!domJid){
      return false;
    }
    oa::XMLUtils().DomstrToStd(jid,domJid);
    return true;
  }
  bool BuildRoster::GetSub(std::string& sub,DOMNode*const nd)
  {
    if(!nd){
      return false;
    }
    boost::scoped_array<XMLCh> attrName(XMLString::transcode("subscription"));
    const XMLCh* domSub = ((DOMElement*)nd)->getAttribute(attrName.get());
    if(!domSub){
      return false;
    }
    oa::XMLUtils().DomstrToStd(sub,domSub);
    return true;
  }
  bool BuildRoster::GetGroup(std::string &group, DOMNode*const nd)
  {
    if(!nd){
      return false;
    }
    if(!nd->hasChildNodes()){
      return false;
    }
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("group"));
    DOMNodeList* groups=((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(groups->getLength()<=0){
      return false;
    }
    DOMNode* ndGroup = groups->item(0);
    if(!ndGroup){
      return false;
    }
    DOMNode* txt = ndGroup->getFirstChild();
    if(!txt){
      return false;
    }

    const XMLCh* domGroup = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(group,domGroup);

    return true;    
  }
};
