/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldtime.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  bool BuildTime::CanBuild(DOMNode *const nd)const
  {
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> queryName(XMLString::transcode("query"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("jabber:iq:time"));
    if(nd->hasChildNodes() &&
       !XMLString::compareString(iqName.get(),nd->getNodeName())){
      DOMNode*query=nd->getFirstChild();
      if(!XMLString::compareString(queryName.get(),query->getNodeName())&&
	 !XMLString::compareString(uri.get(),query->getNamespaceURI())){
	return true;
      }
    }
    return false;
  }
  JabPacket* BuildTime::Build(DOMNode *const nd)
  {
    if(!nd){return 0;}
    std::auto_ptr<JabTime> ptime(new JabTime);
    DOMNode* time = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Time",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }    
#endif
    JabError error;
    if(GetError(error,time)){
      ptime->Error(error);
    }
    const XMLCh* timeName = ((DOMElement*)time)->getTagName();
    std::string strTimeName;
    oa::XMLUtils().DomstrToStd(strTimeName,timeName);
    if(strTimeName != "iq"){
      return 0;
    }

    std::string to,from,id,type,utc,tz,display;
    GetTo(to,time);
    GetFrom(from,time);
    GetId(id,time);
    GetUTC(utc,time);
    GetTimeZone(tz,time);
    GetDisplay(display,time);

    ptime->To(to);
    ptime->From(from);
    ptime->Id(id);
    ptime->Utc(utc);
    ptime->TimeZone(tz);
    ptime->Display(display);
    
    return ptime.release();
  }

  int BuildTime::GetUTC(std::string &sutc,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("utc"));
    DOMNodeList* utcs = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(utcs->getLength()<0){
      return 1;
    }
    DOMNode* utc = utcs->item(0);
    if(!utc || !utc->hasChildNodes()){
      return 1;
    }
    DOMNode* txt = utc->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domUtc = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(sutc,domUtc);
    return 0;
  }
  int BuildTime::GetTimeZone(std::string &stz,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("tz"));
    DOMNodeList *tzs = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(tzs->getLength()<0){
      return 1;
    }
    DOMNode* tz = tzs->item(0);
    if(!tz || !tz->hasChildNodes()){
      return 1;
    }
    DOMNode *txt = tz->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domTz = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(stz,domTz);
    return 0;
  }
  int BuildTime::GetDisplay(std::string &sdisp,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("display"));
    DOMNodeList* displays = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(displays->getLength()<0){
      return 1;
    }
    DOMNode *display = displays->item(0);
    if(!display || !display->hasChildNodes()){
      return 1;
    }
    DOMNode *txt = display->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domDisplay = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(sdisp,domDisplay);
    return 0;
  }
};
