/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMNamedNodeMap.hpp>
#include <xercesc/dom/DOMNodeList.hpp>
#include <xercesc/dom/DOMElement.hpp>
#if !defined WIN32_VC6
/**@todo find a better solution */
#include "oapu/oapr_logstream.hpp"
#endif
#include "framework/xmlutil.hpp"
#include "jabber/jabbldver.h"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  bool BuildVersion::CanBuild(DOMNode *const nd)const
  {
    boost::scoped_array<XMLCh> iqName(XMLString::transcode("iq"));
    boost::scoped_array<XMLCh> queryName(XMLString::transcode("query"));
    boost::scoped_array<XMLCh> uri(XMLString::transcode("jabber:iq:version"));
    if(nd->hasChildNodes() &&
       !XMLString::compareString(iqName.get(),nd->getNodeName())){
      DOMNode*query=nd->getFirstChild();
      if(!XMLString::compareString(queryName.get(),query->getNodeName())&&
	 !XMLString::compareString(uri.get(),query->getNamespaceURI())){
	return true;
      }
    }
    return false;
  }
  JabPacket* BuildVersion::Build(DOMNode *const nd)
  {
    if(!nd){return 0;}
    std::auto_ptr<JabVersion> pversion(new JabVersion);
    DOMNode *ver = nd;
#if !defined WIN32_VC6
/**@todo find a better solution */
    oapr::logstrm<<OAPR_LOG("Building Version",oapr::Log_Info);
    if(oapr::loglvl>=oapr::Log_Debug){
      oapr::logstrm<<oa::XMLUtils().Print(nd);
    }    
#endif
    JabError error;
    if(GetError(error,ver)){
      pversion->Error(error);
    }
    const XMLCh* verName = ((DOMElement*)ver)->getTagName();
    std::string strVerName;
    oa::XMLUtils().DomstrToStd(strVerName,verName);
    if(strVerName!="iq"){
      return 0;
    }
    std::string to,from,id,type,name,version,os;
    GetTo(to,ver);
    GetFrom(from,ver);
    GetId(id,ver);
    GetType(type,ver);
    GetName(name,ver);
    GetVersion(version,ver);
    GetOs(os,ver);

    pversion->To(to);
    pversion->From(from);
    pversion->Id(id);
    pversion->Type(type);
    pversion->Name(name);
    pversion->Version(version);
    pversion->Os(os);


    return pversion.release();
  }

  int BuildVersion::GetName(std::string &sname,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("name"));
    DOMNodeList* names = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(names->getLength()<0){
      return 1;
    }
    DOMNode *name = names->item(0);
    if(!name || !name->hasChildNodes()){
      return 1;
    }
    DOMNode* txt = name->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domName = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(sname,domName);
    return 0;
  }
  int BuildVersion::GetVersion(std::string &sver,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("version"));
    DOMNodeList* vers = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(vers->getLength()<0){
      return 1;
    }
    DOMNode *ver = vers->item(0);
    if(!ver || !ver->hasChildNodes()){
      return 1;
    }
    DOMNode* txt = ver->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domVer = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(sver,domVer);
    return 0;
  }
  int BuildVersion::GetOs(std::string &sos,DOMNode*const nd)
  {
    boost::scoped_array<XMLCh> tagName(XMLString::transcode("os"));
    DOMNodeList *oss = ((DOMElement*)nd)->getElementsByTagName(tagName.get());
    if(oss->getLength()<0){
      return 1;
    }
    DOMNode* os = oss->item(0);
    if(!os || !os->hasChildNodes()){
      return 1;
    }
    DOMNode* txt = os->getFirstChild();
    if(!txt){
      return 1;
    }
    const XMLCh* domOs = txt->getNodeValue();
    oa::XMLUtils().DomstrToStd(sos,domOs);
    return 0;
  }
};
