 /********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "xstrm/jabauth.h"
#include "jabber/jabsesmgr.h"

jabber::SessionMgr *jabber::SessionMgr::p_this=0;

namespace jabber
{
  SessionMgr*const SessionMgr::Instance()
  {
    if(!SessionMgr::p_this){
      SessionMgr::p_this = new SessionMgr;
    }
    return SessionMgr::p_this;
  }
  void SessionMgr::Terminate()
  {
    if(!SessionMgr::p_this){
      return;
    }
    SessionMgr *ez = SessionMgr::p_this;
    ses_cont::iterator spos;
    avail_cont::iterator apos;
    for(spos = ez->m_active.begin(); spos!=ez->m_active.end();++spos){
      spos->second->Shutdown();
    }
    for(apos = ez->m_connected.begin(); apos!=ez->m_connected.end();++apos){
      apos->second->Shutdown();
    }
    
    delete SessionMgr::p_this;
    SessionMgr::p_this=0;
  }

  SessionMgr::jids SessionMgr::GetConnections()
  {
    jids j;

    ses_cont::iterator pos;
    for(pos = m_active.begin(); pos != m_active.end(); ++pos){
      j.push_back(pos->first);
    }
    return j;
  }
  bool SessionMgr::IsConnected(const JabJid& jid)
  {
    ses_cont::iterator active=m_active.find(jid.Jid());
    if(active==m_active.end()){
      /* look in the connections */
      avail_cont::iterator avail=m_connected.find(jid.Domain());
      if(avail==m_connected.end()){return false;}
      else{return avail->second->IsConnected();}
    }
    return active->second->IsConnected();
  }
  bool SessionMgr::IsAuthenticated(const JabJid& jid){
    ses_cont::iterator active=m_active.find(jid.Jid());
    if(active==m_active.end()){
      return false;
    }else{
      return active->second->IsAuthenticated();
    }    
  }
  JabJid SessionMgr::Connect(JabSession::builders_ptr builder,
			     JabSession::eventchain_ptr events,
			     JabAuth *auth,
			     int prt,
			     bool sync)
  {
    //first perform paranoia checks
    if(!builder || !auth){
      OAPR_THROW(oapr::invalid_arg,"Builders/JabAuth are null");
    }
    std::string wjid = auth->Jid(), emptyStr;
    JabJid theJid(wjid),emptyJid(emptyStr);
    //do we have a connection fo the auth jid?
    ses_cont::iterator redo = m_active.find(theJid);
    if(redo == m_active.end()){
      //its ok we make the connection
      boost::shared_ptr<JabSession> sesnew(new JabSession);
      sesnew->Connect(builder,events,auth,prt,sync);
      m_active.insert(std::make_pair(theJid,sesnew));
      return theJid;
    }else{
      OAPR_THROW(oapr::runtime_error,"Session is already established");
      return emptyJid;
    }    
  }
  int SessionMgr::Connect(const std::string &domain, int port)
  {
    /* make a combo domain:port for the map */
    avail_cont::iterator fnd = m_connected.find(domain);
    if(fnd==m_connected.end()){
      boost::shared_ptr<JabSession> ses(new JabSession);
      ses->Connect(domain,port);
      m_connected.insert(std::make_pair(domain,ses));
    }else{
      OAPR_THROW(oapr::runtime_error,"Session is already connected");
    }
    return 0;
  }
  int SessionMgr::Authenticate(JabAuth *auth, bool sync)
  {
    if(!auth){
      OAPR_THROW(oapr::invalid_arg,"Null auth pointer");
    }
    std::string domain = auth->Domain();
    /* find it in m_connected*/
    avail_cont::iterator avail=m_connected.find(domain);
    if(avail == m_connected.end()){
      /* we didn't find one, is it already in the active?*/
      JabJid jid(auth->Jid());
      ses_cont::iterator active=m_active.find(jid);
      if(active == m_active.end()){
	/*It is neither connected or authenticated*/
	OAPR_THROW(oapr::runtime_error,"Session is not connected");
      }else{
	/*Found it in the authenticated, retry to auth*/
	active->second->Authenticate(auth,sync);
	return 0;
      }
      
    }
    /* found it in the connected, so ath it */
    avail->second->Authenticate(auth,sync);
    //    if(avail->second->IsAuthenticated()){
      /* once we auth, move this to active*/
      m_active.insert(std::make_pair(JabJid(auth->Jid()),avail->second));
      m_connected.erase(avail);
      //    }
    return 0;
  }
  int SessionMgr::Disconnect(const JabJid &jid)
  {
    ses_cont::iterator fnd = m_active.find(jid);
    if(fnd == m_active.end()){
      return 0;//nothing here.. so just return
    }
    if(fnd->second){
      try{
	fnd->second->Shutdown();
      }catch(...){}
      m_active.erase(fnd);
    }
    return 0;
  }

  int SessionMgr::Send(const JabJid &jid, JabPacket *pckt)
  {
    ses_cont::iterator fnd = m_active.find(jid);
    if(fnd == m_active.end()){
      OAPR_THROW(oapr::runtime_error,"The JID is not authenticated");
    }else if(!fnd->second){//ok we found something can we use it?
      OAPR_THROW(oapr::runtime_error,"The associated session has been deallocated");
    }else{//finally, send the damned thing
      return fnd->second->Send(pckt);
    }
  }
  int SessionMgr::Send(const JabJid &jid, const std::string &xml)
  {
    ses_cont::iterator fnd = m_active.find(jid);
    if(fnd == m_active.end()){
      OAPR_THROW(oapr::runtime_error,"Unable to find session associated with jid");
    }else if(!fnd->second){
      OAPR_THROW(oapr::runtime_error,"The associated sessoin has been deallocated");
    }else{
      return fnd->second->Send(xml);
    }
    return 0;
  }

  int SessionMgr::GetBuilders(const JabJid &jid, 
			      SessionMgr::builders_ptr &builders)
  {
    ses_cont::iterator ses = m_active.find(jid);
    if(ses == m_active.end()){
      OAPR_THROW(oapr::runtime_error,"Unable to find a valid session");
    }
    ses->second->GetBuilders(builders);
    return 0;
  }
  int SessionMgr::Events(const JabJid &jid, eventchain_ptr &events)
  {
    ses_cont::iterator ses = m_active.find(jid);
    if(ses == m_active.end()){
      OAPR_THROW(oapr::runtime_error,"Unable to find a valid session");
    }
    ses->second->GetEvent(events);
    return 0;
  }
};
