/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABSESMGR_H
#define OAJABBER_JABSESMGR_H
#include "jabber/oajabber_exports.h"
#include <map>
#include <vector>

#include "xstrm/jabjid.h"
#include "xstrm/jabsession.h"

namespace jabber
{
  class OAJABBER_API SessionMgr;

  class SessionMgr
  {
  public:
    /** typedef for list of jabber id*/
    typedef std::vector<JabJid> jids;
    typedef JabSession::builders_ptr builders_ptr;
    typedef JabSession::eventchain_ptr eventchain_ptr;
  public:
    /// Get the single instance
    /**
     * This is the accessor to the instance
     * of the SessionMgr.
     * @return pointer to the instance
     */
    static SessionMgr*const Instance();

    /// Terminate the SessionMgr
    /**
     * Disconnects all the existing sessions
     * and releases all resources allocated
     */
    static void Terminate();

    /// Retrieve list of Jabber IDs
    /** 
     * Each session is associated with a specific and unique
     * Jabber ID. This will return a vector of all the 
     * jids currently  under the control of the SessionMgr
     * @return A vector of JabJid
     */
    jids GetConnections();

    bool IsConnected(const JabJid& jid);
    bool IsAuthenticated(const JabJid& jid);

    ///Establish a connection and authenticate
    /**
     * Creates a JabSession and places it under
     * control of the SessionMgr. See JabSession
     * @param builders Builders 
     * @param events EventChain pointer
     * @param auth Auth pointer
     * @param prt Port to connect on
     * @param sync Whether to make auth synchronous
     * @return the jabber id
     * @throws
     * - oaexc::InvalidArgument Will be thrown when the parameters are null
     * - oaexc::Runtime Will be thrown if the connection is already established
     */
    JabJid Connect(JabSession::builders_ptr builders,
                   JabSession::eventchain_ptr events,
                   JabAuth * auth,
                   int prt,
                   bool sync=true);
    ///Establish a connection
    /**
     * Establishes a tcp connection to the host and port
     * specified
     * @param domain The host to connect to
     * @param port The port to connect on
     * @return zero on success, non-zero otherwise
     * @throws
     * - oaexc::Runtime Will be thrown when an existing session is
     * found.
     */
    int Connect(const std::string &domain, int port);
    ///Authenticate
    /**
     * Authenticate against an established connection
     * with a JabAuth
     * @param auth The authentication object
     * @param sync Whether to make auth synchronous
     * @return zero on success, non-zero otherwise
     * @throws
     * - oaexc::InvalidArgument will be thrown if the auth parameter is null
     * - oaexc::Runtime will be thrown if a connection can not be found
     */
    int Authenticate(JabAuth *auth, bool sync=true);
    /// Disconnect a specific JID
    /**
     * Disconnect the session associated
     * with the jabber id
     * @param jid jabber ID to disconnect
     * @return zero on success, non-zero on failure
     */
    int Disconnect(const JabJid &jid);

    ///Send a packet to the endpoint 
    /**
     * Send a packet to the endpoint, normally a jabber server,
     * associated with the Jabber ID.
     * @param jid Jabber ID 
     * @param pckt Jabpacket to send
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::Runtime will be thrown if there is not an established
     * session, or if the session has been deallocated
     */
    int Send(const JabJid &jid, JabPacket *pckt);

    ///Send a string to the endpoint
    /**
     * Just like previous send, only a string
     * @param jid the jabber id to send from
     * @param xml the string to send
     * @return zero on success, non-zero on otherwise
     * @throws
     * - oaexc::Runtime will be thrown if there is not an established
     * session, or if the session has been deallocated
     */
    int Send(const JabJid &jid, const std::string &xml);

    /// Get the Builder collection for a jid
    /**
     * Retrieve the Builders for a specific JID
     * @param jid The JID to retrieve Builders for
     * @param builders Pointer-pointer for the builders
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::InvalidArgument will be thrown if builders is null
     * or if *builders points to something
     * - oaexc::Runtime will be thrown if a session can not be found
     * for the specified jid
     */
    int GetBuilders(const JabJid &jid, builders_ptr &builders);

    /// Get the EventChain for a jid
    /**
     * Same as GetBuilders except it retrieves the EventChain
     * @param jid The JID
     * @param events EventChain
     */
    int Events(const JabJid &jid, eventchain_ptr &events);
    
    
  protected:
    ///The protected ctor
    SessionMgr(){}
    ///The protected dtor
    virtual ~SessionMgr(){}
  private:
    static SessionMgr *p_this;
  protected:
    typedef boost::shared_ptr<JabSession> elem_type;
    typedef std::map<std::string,elem_type> avail_cont;
    typedef std::map<JabJid,elem_type> ses_cont;
    avail_cont m_connected;
    ses_cont m_active;
    
  };
};
#endif
