/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2004 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include "jabber/register.h"
#include "jabber/jabbldstrm.h"
#include "jabber/ijabregister.h"
#include "jabber/jabbldregister.h"
#include "jabber/jabbldiq.h"
#include "oapr/oapr_exception.hpp"

namespace jabber
{
  const char*const reg_uri="xmpp:xmlns=jabber:iq:register";
  const char*const iq_uri="xmpp:iq";
  const char*const close_stream_str="</stream:stream>";
  class RegEventWrapper : public JabEvent
  {
  public:
    RegEventWrapper(RegisterMgr*const rm)
      : m_rm(rm){}
    ~RegEventWrapper(){}
    std::string Id()const{return "jabber::RegEventWrapper";}
  protected:
    JabEvent::HANDLE_MODES Exec(const JabJid& jid, JabPacket* pckt){
      return m_rm->HandleEvent(jid,pckt);
    }
  private:
    RegisterMgr*const m_rm;
  };

  RegisterMgr::~RegisterMgr()
  {
    if(m_session.get() && m_session->IsConnected()){
      m_session->Shutdown();
    }

  }
  void RegisterMgr::Destroy()
  {
    if(m_session.get() && m_session->IsConnected()){
      m_session->Shutdown();
      m_session.reset();
    }
  }
  void RegisterMgr::Create(const io_ptr& ip, const event_ptr& ep)
  {
    try{
      m_io=ip;
      m_callback=ep;
      m_session.reset(new JabSession);

      if(m_io.get()  && m_session.get()){
        m_state=Register_Init;
      }

      /* deal with adding the event*/
      JabSession::eventchain_ptr echain;
      m_wrapper.reset( new RegEventWrapper(this));
      m_session->GetEvent(echain);
      echain->Subscribe("xmpp:stream",m_wrapper);

      /* add the builder */
      JabSession::builders_ptr bchain;
      StreamBuilder strmbuild;
      m_session->GetBuilders(bchain);
      bchain->Insert(strmbuild);
      
      Connect();
      Stream();
      if(m_sync){
        oapr::mlock lck(m_mutex);
        m_sig.wait(m_mutex);
      }
    }OAPR_CATCH()
  }


  JabEvent::HANDLE_MODES RegisterMgr::Exec(const JabJid& jid, JabPacket* pckt)
  {
    switch(State()){
    case Register_Stream: return Query(jid,pckt);
    case Register_Query: return Request(jid,pckt);
    case Register_Request: return Created(jid,pckt);
      //    case Register_Created: return 
    default: break;
    };

    m_sig.signal();
    return JabEvent::PASS;
  }
  void RegisterMgr::Connect()
  {
    try{
      if(State()!=Register_Init){
        OAPR_THROW(oapr::invalid_state,"Tried to connect without being initialized");
      }
      m_session->Initialize(m_io.get(),m_io.get(),m_jid);
      if(m_session->IsConnected()){
        m_state=Register_Connected;
      }
    }OAPR_CATCH();
  }
  void RegisterMgr::Stream()
  {
    try{
      if(State()!=Register_Connected){
        OAPR_THROW(oapr::invalid_state,"Trying to send data when not connected");
      }
      /**@todo move this out into a factory*/
      std::string stream= std::string("")+
        "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>"
        "<stream:stream to=\""+m_jid.Domain()+"\""
        " xmlns=\"jabber:client\""
        " xmlns:stream=\"http://etherx.jabber.org/streams\""
        ">";
      
      m_session->Send(stream);

      m_state=Register_Stream;
    }OAPR_CATCH();
  }
  JabEvent::HANDLE_MODES RegisterMgr::Query(const JabJid& jid, JabPacket* pckt)
  {
    /*first we determine if there is an error, if so,
     * invoke the error function
     */
    if(pckt->HasError()){
      return Error(jid,pckt);
    }
    JabRegister rqry;
    rqry.Type("get");

    JabSession::eventchain_ptr echain;
    m_session->GetEvent(echain);
    //echain->Unsubscribe("xmpp:stream",m_wrapper->Id());
    echain->Subscribe(rqry.Uri(),m_wrapper);

    JabSession::builders_ptr bchain;
    BuildRegister breg;
    m_session->GetBuilders(bchain);
    bchain->Insert(breg);

    m_state=Register_Query;
    m_session->Send(&rqry);

    return JabEvent::CONTINUE;
  }
  JabEvent::HANDLE_MODES RegisterMgr::Request(const JabJid& jid, JabPacket* pckt)
  {
    if(pckt->HasError()){
      return Error(jid,pckt);
    }

    JabSession::eventchain_ptr echain;
    m_session->GetEvent(echain);
    //echain->Unsubscribe(reg_uri,m_wrapper->Id());
    echain->Subscribe(iq_uri,m_wrapper);

    JabRegister reg;
    reg.Type("set");
    reg.SetValue("username",m_jid.Name());
    reg.SetValue("password",m_key);

    JabSession::builders_ptr bchain;
    BuildIq iqbld;
    iqbld.Id(reg.Id());
    m_session->GetBuilders(bchain);
    bchain->Insert(iqbld);


    
    m_state=Register_Request;
    m_session->Send(&reg);
    return JabEvent::CONTINUE;
  }
  JabEvent::HANDLE_MODES RegisterMgr::Created(const JabJid& jid, JabPacket* pckt)
  {
    if(pckt->HasError()){
      return Error(jid,pckt);
    }
    /* we are created, so set state and signal*/
    m_state=Register_Created;
    m_sig.signal();

    return JabEvent::CONTINUE;
  }
  JabEvent::HANDLE_MODES RegisterMgr::Error(const JabJid& jid, JabPacket* pckt)
  {
    /* verify the rest of the code isn't stupid*/
    if(pckt->HasError()){
      m_hasError=true;
      m_error=pckt->Error();
      m_state=Register_Error;
      m_sig.signal();
    }
    return JabEvent::CONTINUE;
  }
};
