/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/dom_ptr.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_delete.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  std::string PcktDelete::xml()
  {
    std::string ret;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(ret,built.get());
    }
    return ret;
  }
  DOMNode*const PcktDelete::BuildXml()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;
    /* if this is called, its a client implementation
     * since the daemon relies totally on the PcktCont
     */
    /* we build the table element first to allow it to set 
     * the type for IQ to grab if an error happens
     */
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> tableElem(BuildTable());

    /* now we build the database element
     * the namespace is Xmlns() 
     * note: because the formatter sucks atm, we have
     * to manually add the xmlns attribute
     */
    tagName.set("database");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
							tagName.get()));

    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());
    /* the name attribute is required on the database
     * element. throw a logic error if its not there
     */
    std::string dbname=Database();
    if(dbname.empty()){
      OAPR_THROW(oapr::logic_error,"Required database attribute name is empty");
    }

    attrName.set("name");
    attrValue.set(dbname.c_str());
    dbElem->setAttribute(attrName.get(),attrValue.get());


    /* building the IQ element
     */
    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* now we append everything in its proper
     * order
     */
    dbElem->appendChild(tableElem.release());
    iqElem->appendChild(dbElem.release());

    /* set the base m_node*/
    /*
    m_node = iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
  DOMNode*const PcktDelete::BuildTable()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    /* create the table element... always*/
    tagName.set("table");
    oa::dom_ptr<DOMElement> tableElem(doc->createElement(tagName.get()));

    /* set the attribute name on the table element
     * this is required so throw if its empty
     */
    std::string name=Table();
    if(name.empty()){
      OAPR_THROW(oapr::logic_error,"Required attribute name is empty");
    }

    attrName.set("name");
    attrValue.set(name.c_str());
    tableElem->setAttribute(attrName.get(),attrValue.get());

    /* get the type. 
     * - if its of type 'set', then build the where clause
     * - if its of type 'get', then throw an exception
     * - if its of type 'result', then do nothing
     */
    std::string type=Type();
    if(type=="set"){
      /* create the where element
       */
      tagName.set("where");
      oa::dom_ptr<DOMElement> whereElem(doc->createElement(tagName.get()));
      /**\todo there needs to be more error checking here*/
      /* create the columns and append to where
       */
      /**\todo this needs to be moved out into the base class*/
      Where::clause_type::iterator pos = m_where.m_clause.begin();
      for(;pos!=m_where.m_clause.end();++pos){

	/* create the column element*/
	tagName.set("col");
	oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));

	/* ad the name attribute*/
	attrName.set("name");
	attrValue.set(pos->col.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());

	/* add the op attribute*/
	attrName.set("op");
	attrName.set(pos->op.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());
	
	/* if a conjunction exists, add it*/
	if(!pos->conj.empty()){
	  attrName.set("conj");
	  attrValue.set(pos->conj.c_str());
	  colElem->setAttribute(attrName.get(),attrValue.get());
	}

	/* add the value as a text node*/
	tagValue.set(pos->val.c_str());
	oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	colElem->appendChild(colValue.release());
	/* add this to the where elem*/
	whereElem->appendChild(colElem.release());
      }
      /* add the whole where clause mess to the table*/
      tableElem->appendChild(whereElem.release());
    }else if(type=="get"){
      OAPR_THROW(oapr::logic_error,"Type is of value get");
    }else{
      /* now we check if we need to build an error element
       * - we do NOT do this in the set type because thats sending
       *   an error to the component
       * - we throw and exception in get because thats an internal
       *   error to the application
       * - only if we are responding to a request should an error
       *   element be built, regardless of whether an error exists
       */
      if(HasError()){
	Type("error");
	oa::dom_ptr<DOMNode> errorElem(BuildError());
	tableElem->appendChild(errorElem.release());
	/* set the type to error*/

      }
    }
    return tableElem.release();
  }

};
