/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/

#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_insert.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  std::string PcktInsert::xml()
  {
    std::string xml;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(xml,built.get());
    }
    return xml;
  }
  DOMNode*const PcktInsert::BuildXml()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    /* create the table element first
     * to give it a change to generate errors
     * and change the iq type
     */
    oa::dom_ptr<DOMNode> tableElem(BuildTable());

    /* create the database element within the namespace
     */
    tagName.set("database");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
						       tagName.get()));

    /* forcing the xmlns attribute to exist
     * since the formatter sucks
     */
    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());
    /* set the database name
     * this is required so throw a logic error if emtpy
     */
    std::string dbname=Database();
    if(dbname.empty()){
      OAPR_THROW(oapr::logic_error,"Required database attribute name is empty");
    }
    attrName.set("name");
    attrValue.set(dbname.c_str());
    dbElem->setAttribute(attrName.get(),attrValue.get());
    /* create the iq element
     */
    oa::dom_ptr<DOMNode> iqElem(BuildIq());

    /* put the tree together
     */
    dbElem->appendChild(tableElem.release());
    iqElem->appendChild(dbElem.release());
    /*
    m_node = iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
  DOMNode*const PcktInsert::BuildTable()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    /* create the table element */
    //    tagName.reset(XMLString::transcode("table"));
    tagName.set("table");
    oa::dom_ptr<DOMElement> tableElem(doc->createElement(tagName.get()));

    /* set the table name attribute
     * this is required, so throw a logic error if empty
     */
    std::string name=Table();
    if(name.empty()){
      OAPR_THROW(oapr::logic_error,"Required table attribute name is empty");
    }

    //    attrName.reset(XMLString::transcode("name"));
    //    attrValue.reset(XMLString::transcode(name.c_str()));
    attrName.set("name");
    attrValue.set(name.c_str());
    tableElem->setAttribute(attrName.get(),attrValue.get());


    /* get the type and build xml accordingly
     * but first make sure its a valid value for type
     * only allowed values are set,result,error
     * throw logic error otherwise
     */
    std::string type=Type();
    if(type != "set" &&
       type != "result" &&
       type != "error"){
      std::string excmsg = "Type has invalid value of "+type;
      OAPR_THROW(oapr::logic_error,excmsg);
    }

    if(type=="set"){
      /* if type is set, the we have to build the
       * list of columns with values
       */
      colval_type::iterator pos=m_colval.begin();
      for(;pos!=m_colval.end();++pos){
	/* create the col element*/
	//	tagName.reset(XMLString::transcode("col"));
	tagName.set("col");
	oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));

	/* the column name is required
	 * throw a logic error if empty
	 */
	if(pos->name.empty()){
	  OAPR_THROW(oapr::logic_error,"Required col attribute name is empty");
	}
	//	attrName.reset(XMLString::transcode("name"));
	//	attrValue.reset(XMLString::transcode(pos->name.c_str()));
	attrName.set("name");
	attrValue.set(pos->name.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());

	/* the value is not required
	 * cause an empty value indicates null
	 */
	if(!pos->value.empty()){
	  //	  tagValue.reset(XMLString::transcode("col"));
	  tagValue.set("col");
	  oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	  colElem->appendChild(colValue.release());
	}
	// append to table elem
	tableElem->appendChild(colElem.release());
      }
    }
    if(HasError()){
      Type("error");
      oa::dom_ptr<DOMNode> errorElem(BuildError());
      tableElem->appendChild(errorElem.release());
    }
    return tableElem.release();
  }
};
