/********************************************************************
 * OpenAether Framework
 * Contains objects to aid in the development of applications
 * mainly focusing on test applications and daemons
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_schema.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{

  /*
   * see the spec/jep for reference of the xml this builds
   */
  std::string PcktSchema::xml()
  {
    /* What this does is get the xml node
     * and then translates it into a string... which is returned
     * simple eh?
     */
    DOMNode* theXml = BuildXml();
    std::string strXml;
    if(theXml){
      oa::XMLUtils().Print(strXml,theXml);
    }
    return strXml;
  }

  DOMNode*const PcktSchema::BuildXml()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;
    /* as with all buildxml() we do the table first
     * then the iq to allow errors to propagate
     */
    DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> tableElem(BuildTable());

    tagName.set("database");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
							tagName.get()));

    /* again forcing the attribute xmlns to exist
     */
    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());
    /* the name attribute is required on the database
     * element. throw a logic error if its not there
     */
    std::string dbname=Database();
    if(dbname.empty()){
      OAPR_THROW(oapr::logic_error,"Required database attribute name is empty");
    }

    attrName.set("name");
    attrValue.set(dbname.c_str());
    dbElem->setAttribute(attrName.get(),attrValue.get());



    /* now build the iq*/
    oa::dom_ptr<DOMNode> iqElem(BuildIq());
    /* there is a potential for the table elem to be null*/
    if(tableElem.get()){
      dbElem->appendChild(tableElem.release());
    }

    iqElem->appendChild(dbElem.release());
    /*
    m_node = iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
  DOMNode*const PcktSchema::BuildTable()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;
    
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    tagName.set("table");
    oa::dom_ptr<DOMElement> tableElem(doc->createElement(tagName.get()));

    /* the table name is allowed to be empty, 
     * if and only if this is a get request
     * unfortunately, all the if/else branches
     * are final, e.g. they *all* assign to m_node.
     * this breaks from other BuidlTable impls... *sigh*
     */
    std::string name=Table();
    std::string type=Type();
    if(type != "get" &&
       type != "result" &&
       type != "error"){
      /* only allowed types are error, result, get
       * if its none of these types, throw
       */
      std::string excmsg="Type has value of "+type+" which is not allowed";
      OAPR_THROW(oapr::logic_error,excmsg);			  
    }


    if(!name.empty()){
      attrName.set("name");
      attrValue.set(name.c_str());
      tableElem->setAttribute(attrName.get(),attrValue.get());
    }

    if(type =="get" && name.empty()){
      tableElem.reset(0);
    }else if(type=="result"){
      /* if the type is result and the name is empty, thats bad*/
      if(name.empty()){
	OAPR_THROW(oapr::logic_error,"Required table attribute name is empty");
      }
      /* if this is a result, then build all the columns 
       * if any.
       * NOTE: its not an error to have no columsn to return
       */


      /* the permission attribute is required on result types
       */
      std::string perm=Permission();
      if(perm.empty()){
	OAPR_THROW(oapr::logic_error,"Required table attribute permission is empty");
      }

      attrName.set("permission");
      attrValue.set(perm.c_str());
      tableElem->setAttribute(attrName.get(),attrValue.get());

      col_schema::iterator pos=m_schema.begin();
      for(;pos!=m_schema.end();++pos){

	/* create the base col element */
	tagName.set("col");
	oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));

	/* get the column name
	 * this is required, if empty throw a logic error
	 */
	if(pos->name.empty()){
	  OAPR_THROW(oapr::logic_error,"Required col attribute name is empty");
	}

	attrName.set("name");
	attrValue.set(pos->name.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());


	/* get the type
	 * this is also required in a result type
	 */
	if(pos->type.empty()){
	  OAPR_THROW(oapr::logic_error,"Required col attribute type is empty");
	}

	attrName.set("type");
	attrValue.set(pos->type.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());

	/* the size attribute is optional
	 */
	if(!pos->size.empty()){
	  attrName.set("size");
	  attrValue.set(pos->size.c_str());
	  colElem->setAttribute(attrName.get(),attrValue.get());
	}
	/* now append the built column onto the table element*/
	tableElem->appendChild(colElem.release());
      }
    }
    if(type != "get"){
      if(HasError()){
	Type("error");
	oa::dom_ptr<DOMNode> errorElem(BuildError());
	if(!tableElem.get()){
	  return errorElem.release();
	}else{
	  tableElem->appendChild(errorElem.release());
	}
      }
    }
    return tableElem.release();
  }

};
