/********************************************************************
 * OpenAether jabDB
 * Transport to allow jabber access to rdbms 
 *
 * Copyright (C) 2002 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <xercesc/util/XMLString.hpp>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "jda/packet_update.hpp"

XERCES_CPP_NAMESPACE_USE
namespace oajabdb
{
  std::string PcktUpdate::xml()
  {
    std::string xml;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(xml,built.get());
    }
    return xml;
  }
  DOMNode*const PcktUpdate::BuildXml()
  {
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;
    
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    oa::dom_ptr<DOMNode> tableElem(BuildTable());

    /* create the database element
     * force into Xmlns() namespace
     */
    //    tagName.reset(XMLString::transcode("database"));
    //    tagValue.reset(XMLString::transcode(Xmlns().c_str()));
    tagName.set("database");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> dbElem(doc->createElementNS(tagValue.get(),
							tagName.get()));

    /* force xmlns attribute to exist
     */
    oa::XercesXmlns()(dbElem.get(),Xmlns().c_str());

    /* set the database name
     * this attribute is required so throw a logic error
     * if its empty
     */
    std::string dbname=Database();
    if(dbname.empty()){
      OAPR_THROW(oapr::logic_error,"Required database attribute name is empty");
    }
    //    attrName.reset(XMLString::transcode("name"));
    //    attrValue.reset(XMLString::transcode(dbname.c_str()));
    attrName.set("name");
    attrValue.set(dbname.c_str());
    dbElem->setAttribute(attrName.get(),attrValue.get());

    /* buidl the iq*/
    oa::dom_ptr<DOMNode> iqElem(BuildIq());
    /* put the tree together*/
    dbElem->appendChild(tableElem.release());
    iqElem->appendChild(dbElem.release());
    /*
    m_node=iqElem.release();
    return m_node.get();
    */
    return iqElem.release();
  }
  DOMNode*const PcktUpdate::BuildTable()
  {

    //    boost::scoped_array<XMLCh> tagName,tagValue,attrName,attrValue;
    oa::xmlstr<XMLCh,char> tagName,
      tagValue,
      attrName,
      attrValue;

    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    /* create the base table element
     */
    //    tagName.reset(XMLString::transcode("table"));
    tagName.set("table");
    oa::dom_ptr<DOMElement> tableElem(doc->createElement(tagName.get()));

    /* the table name is required in all cases
     * so if its empty throw a logic error
     */
    std::string name=Table();
    if(name.empty()){
      OAPR_THROW(oapr::logic_error,"Required table attribute name is empty");
    }
    //    attrName.reset(XMLString::transcode("name"));
    //    attrValue.reset(XMLString::transcode(name.c_str()));
    attrName.set("name");
    attrValue.set(name.c_str());
    tableElem->setAttribute(attrName.get(),attrValue.get());

    std::string type=Type();
    /* the type must be either
     * set,result,error... throw a logic error if anything else
     */
    if(type != "set" &&
       type != "result" &&
       type != "error"){
      std::string excmsg="Type has invalid value of "+type;
      OAPR_THROW(oapr::logic_error,excmsg);
    }

    if(type == "set"){
      /* iterate through the cols and create
       * the col list
       */
      colval_type::iterator pos=m_colval.begin();
      for(;pos!=m_colval.end();++pos){
	/* create the col element */
	//	tagName.reset(XMLString::transcode("col"));
	tagName.set("col");
	oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));

	/* the name attribute is required
	 * throw a logic error if its empty
	 */
	if(pos->name.empty()){
	  OAPR_THROW(oapr::logic_error,"Required col attribute name is empty");
	}
	//	attrName.reset(XMLString::transcode("name"));
	//	attrValue.reset(XMLString::transcode(name.c_str()));
	attrName.set("name");
	attrValue.set(name.c_str());
	colElem->setAttribute(attrName.get(),attrValue.get());

	/* an empty value indicates null
	 * so an empty value is allowed
	 */
	if(!pos->value.empty()){
	  //	  tagValue.reset(XMLString::transcode(pos->value.c_str()));
	  tagValue.set(pos->value.c_str());
	  oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	  colElem->appendChild(colValue.release());
	}
	tableElem->appendChild(colElem.release());
      }
      if(m_where.m_clause.size()>0){
	/* now to buld the where clause*/
	//	tagName.reset(XMLString::transcode("where"));
	tagName.set("where");
	oa::dom_ptr<DOMElement> whereElem(doc->createElement(tagName.get()));
	/**\todo there needs to be more error checking here*/
	/* create the columns and append to where
	 */
	/**\todo this needs to be moved out into the base class*/
	Where::clause_type::iterator pos = m_where.m_clause.begin();
	for(;pos!=m_where.m_clause.end();++pos){
	
	  /* create the column element*/
	  //	  tagName.reset(XMLString::transcode("col"));
	  tagName.set("col");
	  oa::dom_ptr<DOMElement> colElem(doc->createElement(tagName.get()));
	
	  /* ad the name attribute*/
	  //	  attrName.reset(XMLString::transcode("name"));
	  //	  attrValue.reset(XMLString::transcode(pos->col.c_str()));
	  attrName.set("name");
	  attrValue.set(pos->col.c_str());
	  colElem->setAttribute(attrName.get(),attrValue.get());

	  /* add the op attribute*/
	  //	  attrName.reset(XMLString::transcode("op"));
	  //	  attrValue.reset(XMLString::transcode(pos->op.c_str()));
	  attrName.set("op");
	  attrValue.set(pos->op.c_str());
	  colElem->setAttribute(attrName.get(),attrValue.get());
	
	  /* if a conjunction exists, add it*/
	  if(!pos->conj.empty()){
	    //	    attrName.reset(XMLString::transcode("conj"));
	    //	    attrValue.reset(XMLString::transcode(pos->conj.c_str()));
	    attrName.set("conj");
	    attrValue.set(pos->conj.c_str());
	    colElem->setAttribute(attrName.get(),attrValue.get());
	  }

	  /* add the value as a text node*/
	  //	  tagValue.reset(XMLString::transcode(pos->val.c_str()));
	  tagValue.set(pos->val.c_str());
	  oa::dom_ptr<DOMText> colValue(doc->createTextNode(tagValue.get()));
	  colElem->appendChild(colValue.release());
	  /* add this to the where elem*/
	  whereElem->appendChild(colElem.release());
	}
	tableElem->appendChild(whereElem.release());
      }
    }

    if(HasError()){
      Type("error");
      oa::dom_ptr<DOMNode> errorElem(BuildError());
      tableElem->appendChild(errorElem.release());
    }
    return tableElem.release(); 
  }

};
