/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber pubsub library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_PUBSUB_PSENTITY_HPP
#define OAJABBER_PUBSUB_PSENTITY_HPP
#include "xstrm/jabjid.h"
#include "pubsub/psaffiliation.hpp"
#include "pubsub/pssubscription.hpp"

namespace jabber
{
  /**
   * PSEntity encapsulates the entity data. It simply
   * stores the info for an entity, jid, affiliation,
   * and subscription status.
   *
   * \addtogroup EnumTypes
   */
  class PSEntity
  {
  public:
    PSEntity()
      : m_affiliation(PSAffiliation::AT_None),
        m_subscription(PSSubscription::ST_None){}
    PSEntity(const PSEntity &pse)
      : m_nodeName(pse.m_nodeName),
        m_jid(pse.m_jid),
        m_affiliation(pse.m_affiliation),
        m_subscription(pse.m_subscription){}
    ~PSEntity(){}

    /**
     * Set the node name for this entity. This should only be used
     * in affiliations, and *not* with regular entity checks.
     * @param nn name of the node
     */
    void NodeName(const std::string &nn){m_nodeName=nn;}
    /**
     * Get the node name for this entity. This should only be checked
     * in affiliations, and *not* with regular entity checks.
     * @return Name of the node.
     */
    std::string NodeName()const {return m_nodeName;}

    /**
     * Set the jid associated with this entity. It does *not* have to be
     * a full jid, i.e. user@domain/resource. It simply has to be valid.
     * @param jj JabJid for the jid of this entity.
     */
    void Jid(const JabJid &jj){m_jid=jj;}
    /** Get the jid associated with this entity. It does *not* have to be
     * a full/complete jid, merely valid. 
     * @return JabJid jabber id of the entity
     */
    JabJid Jid()const{return m_jid;}

    /**
     * Set the affiliation of this entity. The affiliation is restricted to
     * types in the Affiliation class (which can be converted to string).
     * @param psa The affiliation value of the entity
     */
    void Affiliation(const PSAffiliation& psa){m_affiliation=psa;}
    /**
     * Get the affiliation of this entity. 
     * @return PSAffiliation affilialtion type of the entity.
     */
    PSAffiliation Affiliation()const{return m_affiliation;}

    /**
     * Set the subscription type of the entity.
     * @param ps PSSubscription value for the entity
     */
    void Subscription(const PSSubscription &ps){m_subscription=ps;}
    /**
     * Get the subscription type of the entity
     * @return PSSubscription type of the entity
     */
    PSSubscription Subscription()const{return m_subscription;}
  private:
    std::string m_nodeName;/**< node name */
    JabJid m_jid;/**< JabJid jabber id*/
    PSAffiliation m_affiliation;/**< affiliation */
    PSSubscription m_subscription;/**< subscription */
  };
};
#endif
