/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <iostream>
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapr/oapr_time.hpp"
#include "xstrm/iojabber.h"


namespace jabber
{
  BinTcpInputStream::BinTcpInputStream(istream_type *ts,
				       const XMLByte* const initData, 
				       const unsigned int capacity,
				       const BufOpts bufOpt)
    : m_buffer(0),
      m_bufopts(bufOpt),
      m_capacity(capacity),
      m_curindex(capacity),
      m_lasttcp(0)
  {
    //if specified... take ownership of buffer.
    if(m_bufopts == BufOpt_Copy){
      XMLByte* tmpBuf = new XMLByte[m_capacity];
      memcpy(tmpBuf, initData, capacity);
      m_buffer = tmpBuf;
    }
    //otherwise, just point to it
    else{
      m_buffer = initData;
    }
    m_istrm = ts;
  }
  BinTcpInputStream::~BinTcpInputStream()
  {
    //if owning the buffer...
    if(m_bufopts == BufOpt_Copy){
      delete [] const_cast<XMLByte*>(m_buffer);
    }
    
  }
  unsigned int BinTcpInputStream::isPending(){
    int avail = (m_lasttcp-m_curindex);
    return (avail<=0) ? 0 : avail;
  }
  unsigned int BinTcpInputStream::curPos() const{
    return m_curindex;
  }
  unsigned int BinTcpInputStream::readBytes(XMLByte* const toFill, 
					    const unsigned int maxToRead)
  {
    if(!m_istrm->is_open()){
      //      OAPR_THROW(oapr::runtime_error,"stream is not established");
      return 0;
    }

    int avail = (m_lasttcp - m_curindex);
    //if nothing available... then read from tcpstream
    if(avail<=0){
      avail = 0;
      //      memset((void*)m_buffer,0,m_capacity);
      //      memset((void*)toFill,0,maxToRead);
      //relying on read() to block if nothing.
      //if non-blocking, then saxparser crashes!! @%@#$#@ <-- nasty words
      while(avail==0){       
	try{
	  m_lasttcp = avail = m_istrm->read((char*)m_buffer, m_capacity);
	  if(avail==0){
	    oapr::sleep(100);
	  }
	}catch(const oapr::oapr_exception& oae){
	  if(oae.code()==APR_TIMEUP){
	    continue;
	  }else{
	    return 0;
	  }
	}catch(...){
	  return 0;
	}
      }
      //reset index to beginning
      m_curindex = 0;
    }
    /*
    if(!m_istrm->is_open()){
      OAPR_THROW(oapr::runtime_error,"invalidated stream");
    }
    */
    if(avail < 0){
      //this happens with the read fails
      /* wonder if this is necessary since if read fails 
       * isConnected will return false
       */
      //OAPR_THROW(oapr::runtime_error,"no bytes available on input stream");
      return 0;
    }

    unsigned int toRead = static_cast<unsigned int>(avail) < maxToRead ? avail : maxToRead;

    memcpy(toFill,&m_buffer[m_curindex],avail);
    m_curindex+=avail;

    return toRead;
  }

  void BinTcpInputStream::reset(){
    m_curindex = 0;
  }

  TcpInputSource::TcpInputSource(istream_type *ts, unsigned int cb)
    : m_cb(cb)
  {
    m_istrm = ts;
    p_buffer = new char[cb];
    memset(p_buffer,0,cb);
  }
  TcpInputSource::~TcpInputSource()
  {
    if(p_buffer){
      delete [] p_buffer;
    }
  }

  unsigned int TcpInputSource::isPending(){
    return (p_tcpbin) ? p_tcpbin->isPending() : 0;
  }

  xercesc::BinInputStream* TcpInputSource::makeStream() const
  {
    BinTcpInputStream **temp = (BinTcpInputStream**)&p_tcpbin;
    *temp = new BinTcpInputStream(m_istrm, (const XMLByte*)p_buffer,m_cb);
    return  p_tcpbin;
  }


};//jabber ns
