/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_IOJABBER_H
#define OAJABBER_IOJABBER_H
#include <boost/smart_ptr.hpp>
#include <xercesc/sax/InputSource.hpp>
#include <xercesc/util/BinInputStream.hpp>
#include <xercesc/framework/XMLPScanToken.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include "oapr/oapr_io.hpp"

namespace jabber
{



  class BinTcpInputStream : public xercesc::BinInputStream
  {
  public:
    /**@todo make this typedef more flexible*/
    typedef oapr::basic_io istream_type;
  public:
    
    /// Buffer handling options on ctor
    /**
     *  Options specifying how to handle the buffer passed to the ctor
     */
    enum BufOpts
    {
      BufOpt_Adopt    /**< Does not copy the buffer, but will delete on dtor*/
      , BufOpt_Copy   /**< Copies the buffer, does not reference original*/
      , BufOpt_Reference/**< Use original buffer, do not delete*/
    };
    /// The one and only big ctor
    /**
     * This class is tightly linked with TcpInputSource.
     * In that TcpInputSource is what creates it.
     * This is returned when TcpInputSource::makeStream is called
     * The buffer in TcpInputSource is passed, but is not used!
     * its just for compatibility with XercesC
     * A new buffer is constructed and the curindex if forced to the end
     * This is the wonderful affect of forcing a call to read() on the 
     * tcpstream in the first parse.
     * If you derive your own InputSource to create this:
     * capacity and initData are the only flexible params.
     * ts MUST point to a valid and connected stream
     * bufOpt MUST == BufOpt_Copy Again most of this is provided for 
     * simple compatibility with XercesC
     *
     * @param ts A pointer to a tcp stream
     * @param initData A pointer to a buffer that is used by this.
     * @param capacity Max number of bytes to read/create
     * @param bufOpt What to do with buffer @see BufOpts
     */
    BinTcpInputStream(istream_type *ts, const XMLByte* const initData, 
		      const unsigned int capacity,                            
		      const BufOpts bufOpt = BufOpt_Copy);
    /// The one and only dtor
    /**
     * This will delete the buffer, if and only if this 
     * was constructed with BufOpt_Copy or BufOpt_Adopt
     * as the param for BufOpt. Otherwise, the buffer is 
     * ignored and the dtor does nothing. (literally)
     */
    virtual ~BinTcpInputStream();
    
    /// Returns current position of index
    /**
     * Quite pointless actually.
     * Unless the SAXParser uses this.
     * @return The current value of m_curindex
     */
    unsigned int curPos() const;
    /// Read bytes in the buffer up to maxToRead
    /**
     * Read bytes from teh buffer and place into toFill.
     * Will read up to maxToRead or the max in buffer.
     * If the current index position is at the end of the data,
     * (not the buffer). A call to read is made on the tcp stream.
     * 
     * @return Number of bytes left to read.
     */
    virtual unsigned int readBytes(XMLByte* const toFill, const unsigned int maxToRead);
    /// Resets the current index only
    /**
     * Basically sets current index to zero
     */
    void reset();
    /// Indicates whether there is data left to read
    /**
     * If readBytes() has not read all data in buffer, 
     * this will return the number of bytes left to read.
     * Otherwise it will return zero
     */
    unsigned int isPending();
    
  protected:
    /** Pointer to the tcp stream.*/
    //    oa::smtcpstream     *p_tcpstrm; 
    istream_type *m_istrm;
    /** Buffer to read/write to*/
    const XMLByte   *m_buffer; 
    /** Buffer options, affect dtor/ctor*/
    BufOpts         m_bufopts; 
    /** The maximum capacity of m_buffer*/
    unsigned int    m_capacity; 
    /** Last position read in m_buffer*/
    unsigned int    m_curindex; 
    /** Size of last tcp read operation.*/
    unsigned int    m_lasttcp;  
    
  };

  /// The input source for the SAXParser instance
  /**
   * This is the input source for the SAXParser object
   * You should only use this for progressive scans
   * and NOT if you are expecting a complete chunk of 
   * XML!!!
   *
   * See XercesC doc on parseFirst() parseNext()
   * Or look in TcpThrd::Run() for an example
   *
   */
  class TcpInputSource : public xercesc::InputSource
  {
  public:
    /**@todo make this typedef more flexible*/
    typedef oapr::basic_io istream_type;
  public:
    ///The single ctor
    /**
     * This constructs the input source for
     * the SAXParser.  ts is used to just pass off to 
     * the BinTcpInputStream, and cb is to create the 
     * initial size buffer
     *
     * @param ts A connected, valid tcp stream
     * @param cb the size in bytes of the buffer
     */
    TcpInputSource(istream_type *ts, unsigned int cb=512);
    ///the dtor
    /**
     * does this object own any resources to delete??
     */
    virtual ~TcpInputSource();
    
    /**
     * Allocates a TcpBinInputStream for SAXParser
     * Does the parser own the stream???? yes
     *
     * @return BinInputStream* 
     */
    xercesc::BinInputStream* makeStream() const;
    
    /**
     * Wrapper for tcpstream::isPending()
     * returns non-zero if data is in the tcp buffer
     * otherwise zero is returned
     *
     * @return non-zero if data is in buffer, zero otherwise
     */
    virtual unsigned int isPending();
    
  protected:
    /** The input stream that is owned by SAXParser */
    BinTcpInputStream *p_tcpbin;
    /** initial buffer to transfer to SAXParser */
    char                *p_buffer;
    /** size of p_buffer */
    const unsigned int  m_cb;
    /** the one and only tcpstream */
    istream_type *m_istrm;
  };

};

#endif
