/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <boost/smart_ptr.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/dom/DOMElement.hpp>
#include "oapr/oapr_pool_mgr.hpp"
#include "framework/xerces_xmlns.hpp"
#include "framework/xmlutil.hpp"
#include "framework/xmlstr.hpp"
#include "framework/dom_ptr.hpp"
#include "xstrm/jabjid.h"
#include "xstrm/jabshahash.h"
#include "xstrm/jabauth.h"


XERCES_CPP_NAMESPACE_USE
namespace jabber
{
  std::string JabAuth::Id() const {
    if(m_id.empty()){
      oa::XMLUtils().Uuid(m_id);
    }
    return m_id;
  }
  std::string JabAuth::xml(DOMNode*const nd)
  {
    std::string xml;
    oa::dom_ptr<DOMNode> built(BuildXml(nd));
    if(built.get()){
      oa::XMLUtils().Print(xml,built.get());
    }
    return xml;
  }
  std::string JabAuth::xml()
  {
    std::string xml;
    oa::dom_ptr<DOMNode> built(BuildXml());
    if(built.get()){
      oa::XMLUtils().Print(xml,built.get());
    }
    return xml;
  }

  JabAuthClient::JabAuthClient(std::string jid, std::string pwd)
    : JabAuth("authclient"), 
    m_jid(jid), 
    m_pwd(pwd), 
    m_namespace("jabber:client"){
    JabJid::domain(m_domain,m_jid);
  }
  JabAuthClient::JabAuthClient(const JabAuthClient &jc)
    : JabAuth(jc),
      m_domain(jc.m_domain),
      m_jid(jc.m_jid),
      m_pwd(jc.m_pwd),
      m_namespace(jc.m_namespace){}

  std::string JabAuthClient::Stream() const
  {
    std::string strm;

    strm = "<?xml version=\"1.0\" encoding=\"UTF-8\" ?><stream:stream";
    strm += " to=\""+Domain()+"\"";
    strm += " xmlns=\""+Namespace()+"\"";
    strm += " xmlns:stream=\"http://etherx.jabber.org/streams\"";
    strm += ">";

    return strm.c_str();
  }
  void JabAuthClient::Jid(std::string jid){
    m_jid=jid;
    JabJid::domain(m_domain,m_jid);
  }
  DOMNode*const JabAuthClient::BuildXml(DOMNode*const nd){
    return BuildXml();
  }

  DOMNode*const JabAuthClient::BuildXml()
  {
    oa::xmlstr<XMLCh,char> tagName,tagValue,attrName,attrValue;
    DOMDocument *doc=oa::XMLUtils().Document(DocUri(),"oa");

    tagName.set("iq");
    oa::dom_ptr<DOMElement> iqElem(doc->createElement(tagName.get()));

    attrName.set("type");
    attrValue.set(Type().c_str());
    iqElem->setAttribute(attrName,attrValue);

    attrName.set("id");
    attrValue.set(Id().c_str());
    iqElem->setAttribute(attrName,attrValue);

    tagName.set("query");
    tagValue.set(Xmlns().c_str());
    oa::dom_ptr<DOMElement> queryElem(doc->createElementNS(tagValue,tagName));


    oa::XercesXmlns()(queryElem.get(),Xmlns().c_str());
    /**\todo move this out somewhere to someplace for someone*/
    tagValue.set("http://www.w3.org/200/xmlns");



    tagName.set("username");
    oa::dom_ptr<DOMElement> usernameElem(doc->createElement(tagName));

    std::string username;
    JabJid::uname(username,m_jid);
    tagValue.set(username.c_str());
    oa::dom_ptr<DOMText> usernameValue(doc->createTextNode(tagValue));
    usernameElem->appendChild(usernameValue.release());
    queryElem->appendChild(usernameElem.release());

    tagName.set("resource");
    oa::dom_ptr<DOMElement> resourceElem(doc->createElement(tagName));

    std::string resource;
    JabJid::resource(resource,m_jid);
    tagValue.set(resource.c_str());
    oa::dom_ptr<DOMText> resourceValue(doc->createTextNode(tagValue));
    resourceElem->appendChild(resourceValue.release());
    queryElem->appendChild(resourceElem.release());

    tagName.set("password");
    oa::dom_ptr<DOMElement> passElem(doc->createElement(tagName));

    tagValue.set(m_pwd.c_str());
    oa::dom_ptr<DOMText> passValue(doc->createTextNode(tagValue));
    passElem->appendChild(passValue.release());
    queryElem->appendChild(passElem.release());

    iqElem->appendChild(queryElem.release());
    return iqElem.release();
  }

  JabAuthTrans::JabAuthTrans(std::string srv, 
			     std::string pwd)
    : JabAuth("authtrans"),
      m_done(false),
      m_server(srv), 
      m_secret(pwd),
      m_namespace("jabber:component:accept"),
      m_mutex(oapr::pool_mgr::instance()->unique()){}

  JabAuthTrans::JabAuthTrans(const JabAuthTrans& jt)
    : JabAuth(jt),
      m_server(jt.m_server),
      m_secret(jt.m_secret),
      m_namespace(jt.m_namespace),
      m_mutex(oapr::pool_mgr::instance()->unique()){}

  std::string JabAuthTrans::Stream() const
  {
    /**\todo make this use DOM instead of string.. maybe? */
    std::string strm;

    strm = "<stream:stream ";
    strm += "xmlns=\""+m_namespace+"\" ";
    strm += "stream:xmlns=\"http://etherx.jabber.org/streams\" ";
    strm += "to=\""+m_server+"\">";

    return strm.c_str();
  }
  DOMNode*const JabAuthTrans::BuildXml(DOMNode*const nd)
  {
    oa::xmlstr<XMLCh,char> attrName("id");
    oa::xmlstr<char,XMLCh> attrVal;
    if(!nd){return 0;}
    if(nd->getNodeType() == DOMNode::ELEMENT_NODE){
      DOMElement*const tmp=(DOMElement*const)nd;
      if(tmp->hasAttributes()){
	const XMLCh*idval= tmp->getAttribute(attrName);
	if(idval){
	  attrVal.set(idval);
	  m_id = attrVal.get();
	}
      }
    }

    return BuildXml();
  }

  DOMNode*const JabAuthTrans::BuildXml()
  {
    lock_type lock(m_mutex);
    if(!m_done){
      if(m_id.empty() || m_secret.empty()){
	return 0;
      }
      oa::xmlstr<XMLCh,char> tagName,tagValue;
      SHAHasher sha;
      std::string hash, tohash,shake;

      tohash = m_id+m_secret;
      sha.hash(tohash.c_str(), tohash.length());
      hash = sha.toString();

      tagName.set("handshake");
      tagValue.set(hash.c_str());
      DOMDocument *doc = oa::XMLUtils().Document(DocUri(),"oa");
      oa::dom_ptr<DOMElement> shakeElem(doc->createElement(tagName));
      oa::dom_ptr<DOMText> hashValue(doc->createTextNode(tagValue));
      shakeElem->appendChild(hashValue.release());
      m_done=true;
      return shakeElem.release();
    }else{
      return 0;
    }
  }
};
