/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "boost/smart_ptr.hpp"
#include "xstrm/jabjid.h"


namespace jabber
{
  /************************************************
   *
   *   JabJid
   *
   ************************************************/
  JabJid::JabJid(const JabJid &jj)
  {
    m_jid = jj.m_jid;
    m_name = jj.m_name;
    m_domain = jj.m_domain;
    m_resource = jj.m_resource;
  }
  bool JabJid::IsNull() const{
    if(m_jid.empty()){
      return true;
    }else{
      return false;
    }
  }
  bool JabJid::IsValid() const{
    /* things this can be
     * - can only have a domain (foobar.com)
     * - if has node must have domain (node@foobar.com)
     * - if has resource must have node and domain (node@foobar.com/here)
     */
    if(!m_resource.empty()){
      if(m_domain.empty() || m_name.empty()){
	return false;
      }
    }else if(!m_name.empty()){
      if(m_domain.empty()){
	return false;
      }
    }else if(m_domain.empty()){
      return false;
    }
    return true;
  }
  std::string JabJid::Jid() const{
    return m_jid;
  }
  std::string JabJid::Bare() const{
    JabJid::uname(m_name,m_jid);
    JabJid::domain(m_domain,m_jid);
    return m_name+"@"+m_domain;
  }
  std::string JabJid::Name() const{
    JabJid::uname(m_name,m_jid);    
    return m_name;
  }
  std::string JabJid::Domain() const{
    JabJid::domain(m_domain,m_jid);
    return m_domain;
  }
  std::string JabJid::Resource() const{
    JabJid::resource(m_resource,m_jid);
    return m_resource;
  }
  void JabJid::Jid(const std::string &jid){
    m_jid = jid;
    JabJid::uname(m_name,m_jid);
    JabJid::domain(m_domain,m_jid);
    JabJid::resource(m_resource,m_jid);
  }
  void JabJid::Name(const std::string &name){
    m_name = name;
    MakeJid();
  }
  void JabJid::Domain(const std::string &domain){
    m_domain = domain;
    MakeJid();
  }
  void JabJid::Resource(const std::string &rsrc){
    m_resource = rsrc;
    MakeJid();
  }

  void JabJid::MakeJid(){
    if(m_name.empty()){
      m_jid=m_domain;
    }else{
      m_jid=m_name+"@"+m_domain;
    }
    if(!m_name.empty() && !m_resource.empty()){
      m_jid+="/"+m_resource;
    }
    //    m_jid = m_name + "@" + m_domain + "/" + m_resource;
  }
  int JabJid::uname(std::string &usr, const std::string &jid)
  {
    std::string::size_type begI, endI;
    std::string delim("@"), ret;
    usr.clear();    
    begI = jid.find_first_not_of(delim);
    if(begI == std::string::npos){
      return 0;
    }

    endI = jid.find_first_of(delim,begI);
    if(endI == std::string::npos){
      endI = jid.length();
    }
    
    for(unsigned int i=begI; i<endI; ++i){
      ret += jid[i];
    }
    usr = ret;
    return ret.length();
  }
  int JabJid::domain(std::string &dmn, const std::string &jid)
  {
    std::string::size_type begI, endI;
    std::string at("@"), sl("/");
    dmn.clear();

    begI = jid.find_first_of(at);
    if(begI == std::string::npos){
      return 0;
    }
    
    endI = jid.find_first_of(sl,begI);
    if(endI == std::string::npos){
      endI = jid.length();
    }
    
    begI++;
    for(unsigned int i=begI; i<endI; ++i){
      dmn += jid[i];
    }    
    return dmn.length();
  }
  int JabJid::resource(std::string &rsc, const std::string jid)
  {
    std::string::size_type begI, endI;
    std::string delim("/"), ret;
    rsc.clear();
    
    begI = jid.find_first_of(delim);
    if(begI == std::string::npos){
      return 0;
    }
    
    endI = endI = jid.length();
    
    begI++;
    for(unsigned int i=begI; i<endI; ++i){
      ret += jid[i];
    }
    rsc = ret;
    return ret.length();
  }
  bool JabJid::operator == ( const jabber::JabJid& jid) const 
  {
    return (Jid() == jid.Jid());
  }
  bool JabJid::operator < ( const jabber::JabJid& jid) const
  {
    return (Jid() < jid.Jid());
  }
};

