/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABPACKET_H
#define OAJABBER_JABPACKET_H
#include <xercesc/dom/DOMNode.hpp>
#include "framework/dom_ptr.hpp"
#include "xstrm/xmlinfo.h"
#include "xstrm/ijaberror.h"
#include "xstrm/jabjid.h"
#include "xstrm/ijabx.h"



namespace jabber
{
  class OAXSTRM_API JabPacket;
  /** @defgroup gJabPacket Jabber Packets
   * Core data to route through application
   * @{
   */

  /**
   * JabPacket is the base class for all objects to be created
   * from the input stream from the jabber server. If an object 
   * needs to be created from the xml stream, then it _MUST_ be
   * inherited from JabPacket
   * @author Justin Kirby <justink@rochester.rr.com>
   */
  class JabPacket : public jabber::XmlInfo
  {
  public:
    typedef JabX x_type;
    typedef boost::shared_ptr<x_type> x_ptr;
  public:
    /** @name Contructors */
    //@{

    ///ctor
    /** 
     * Constructs the basic packet with core infor
     * @param tag The root element that this packet 
     *            handles. Usually one of the base
     *            types, <message/> <presence/> <iq/>
     * @param classid A unique ID for this class, used
     *                to prevent constant use of dynamic_cast<>
     */
    JabPacket(std::string tag, std::string classid)
      : XmlInfo(),
        m_hasError(false),
        m_classid(classid),
        m_tag(tag)
    {}
    
    ///copy ctor
    /**
     * This copies all relevant info 
     * There is one issue in that it clones the 
     * DOMNode pointer which this contains. There
     * is the potential for this to throw an exception
     */
    JabPacket(const JabPacket &jp)
      : XmlInfo(jp),
        m_error(jp.m_error),
        m_hasError(jp.m_hasError),
        m_to(jp.m_to),
        m_from(jp.m_from),
        m_id(jp.m_id),
        m_type(jp.m_type),
        m_xmlns(jp.m_xmlns),
        m_classid(jp.m_classid),
        m_tag(jp.m_tag)
    { }
    //@}

    /** @name Destructor */
    //@{
    ///dtor
    virtual ~JabPacket(){}
    //@}

    /** @name Getters */
    //@{
    
    ///Destination Jabber ID 
    virtual JabJid To()const{return m_to;}
    
    ///Source Jabber ID
    virtual JabJid From()const{return m_from;}
    
    ///Unique ID of the packet
    virtual std::string Id()const{return m_id;}
    
    ///The type of packet
    virtual std::string Type()const{return m_type;}

    ///Get the xml namespace
    virtual std::string Xmlns()const{return m_xmlns;}
    //@}

    virtual x_ptr Extension()const{
      return x_ptr(reinterpret_cast<x_type*>(0));
    }

    /** @name Setters */
    //@{

    ///Set the destination JID
    virtual void To(const JabJid &to){m_to = to;}	

    ///Set the source JID
    virtual void From(const JabJid &from){m_from = from;}

    ///Set the ID number
    virtual void Id(const std::string &id){m_id = id;}

    ///Set the type
    virtual void Type(const std::string &type){m_type = type;}

    ///Set the xml namespace
    virtual void Xmlns(const std::string& xmlns){m_xmlns=xmlns;}
    //@}

    virtual void Extension(const x_ptr& xp){}

    ///Get the URI of the packet
    virtual const char*const Uri()const=0;

    /** @name Meta information */
    //@{

    ///Return the tag this packet contains
    /**
     * Each packet handles a specific root element.
     * The elements do not have to be unique, usually
     * the typical jabber elements, message, iq, presence
     * @return The element this packet handles
     */
    virtual std::string Handle()const{return m_tag;}

    ///For the document getting
    /**
     */
    /*
    virtual std::string DocUri()const{
      return "http://www.openaether.org/jabber";
    }
    */


    ///The unique ID for this class
    std::string ClassId()const{return m_classid;}
    //@}


    /** @name Error Information */
    //@{
    ///Whether this packet has an error
    /**
     * Indicates whether an error has occurred
     * with this packet
     * @return true if there is an error, false otherwise
     */
    virtual bool HasError()const{return m_hasError;}

    ///Get the error
    /**
     * Returns JabError containing the error
     * information.
     * @return a JabError
     */
    virtual JabError Error()const{return m_error;}

    ///Set the error information
    /**
     * Assign this packet an error when one occurrs
     * @param je A JabError containing the error information
     */
    virtual void Error(const JabError &je){
      m_hasError = true;
      m_error = je;
    }
    //@}

    /** @name XML Builders */
    //@{

    ///Get the xml in string form
    /**
     * Returns a string containing the xml
     * used primarily in the JabSession
     * @return xml in string
     */
    //    virtual std::string xml()=0;

    ///Get the xml as a domnode
    /**
     * Returns the packet as a DOMNode
     * @return DOMNode pointer
     */
    //    virtual DOMNode*const BuildXml()=0;
    //@}

  protected:
    JabError m_error;
    bool m_hasError;
    JabJid m_to;
    JabJid m_from;
    mutable std::string m_id;
    std::string m_type;
    std::string m_xmlns;
    //    oa::dom_ptr<DOMNode> m_node;
  private:
    std::string m_classid;
    std::string m_tag;
  };

  /** @} */ //end gJabPacket
};

#endif
