/********************************************************************
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#include "framework/first.h"
#include "openaether.hpp"
#include <sstream>
#include "boost/smart_ptr.hpp"
#include <xercesc/util/PlatformUtils.hpp>
#include <xercesc/sax2/XMLReaderFactory.hpp>
#include "framework/xmlutil.hpp"
#include "xstrm/jabjid.h"
#include "xstrm/jabauth.h"
#include "xstrm/jabsession.h"
#include "xstrm/jabbldauth.h"
#include "oapr/oapr_exception.hpp"
#include "oapr/oapr_exc_utils.hpp"
#include "oapr/oapr_time.hpp"
#include "oapr/oapr_io_socket.hpp"
#include "oapr/oapr_debug.hpp"

XERCES_CPP_NAMESPACE_USE
namespace jabber
{

  JabSession::JabSession()
    : m_triedAuth(false),
      m_tcpConnect(false),
      m_jabAuth(BuildAuth::Auth_Unknown),
      m_handler(0),
      p_parser(0),
      p_sigsync(0),      
      m_istream(0),
      m_ostream(0),
      m_delIstream(false),
      m_delOstream(false),
      m_builders(0),
      p_events(0)
  {
  }

  JabSession::~JabSession()
  {
    try{
      //      Shutdown();
      if(m_delOstream && m_ostream){delete m_ostream;}
      if(m_delIstream && m_istream){delete m_istream;}
      if(m_handler){delete m_handler;}
      if(p_parser){delete p_parser;}
    }catch(...){
      //catching it
    }
  }

  void JabSession::GetBuilders(JabSession::builders_ptr &builders)
  {
    if(!m_builders){
      m_builders=new Builders;
    }
    builders=m_builders;
  }
  void JabSession::GetEvent(eventchain_ptr &event){
    if(!p_events){
      p_events = new EventChain;
    }
    event = p_events;
  }
  JabJid JabSession::GetJid(){
    return m_jid;
  }
  void JabSession::OnEvent(JabPacket *pckt)
  {
    if(pckt && p_events){
      p_events->Publish(m_jid,pckt);
    }
  }

  int JabSession::Send(JabPacket *pckt)
  {
    if(!pckt){
      OAPR_THROW(oapr::invalid_arg,"null packet pointer");
    }
    /*
    DOMDocument* doc=oa::XMLUtils().Document(m_outuri,"oa");
    DOMNode*nd=pckt->BuildXml();
    std::string out=oa::XMLUtils().Print(nd);
    doc->appendChild(nd);
    */
    std::string out(pckt->xml());
    return Send(out);
  }
  int JabSession::Send(const std::string &xml)
  {
    if(xml.empty()){
      OAPR_THROW(oapr::invalid_arg,"empty string");
    }

    //check to see if we are connected
    if(!m_ostream || !m_ostream->is_open()){
      OAPR_THROW(oapr::runtime_error, "streams are not established");
    }
    oapr::debug::write("[OUT]");
    oapr::debug::write(xml);
    m_ostream->write(xml.c_str(),xml.length());
    return 0;    
  }
  
  int JabSession::Connect(JabSession::builders_ptr builders,
			  JabSession::eventchain_ptr event,
			  JabAuth *pckt, 
			  int prt,
			  bool sync)
  {
    if(!pckt){
      OAPR_THROW(oapr::invalid_arg,"null authentication packet");
    }

    if(!m_builders){
      m_builders=builders;
    }else{
      OAPR_THROW(oapr::invalid_arg,"Builder is already set");
    }
    if(!p_events){
      p_events=event;
    }else{
      OAPR_THROW(oapr::invalid_arg,"EventChain is already set");
    }

    int rv(0);
    rv = Connect(pckt->Domain(),prt);
    if(rv){return rv;}

    rv = Authenticate(pckt,sync);
    return rv;
  }
  int JabSession::Connect(const std::string domain, int port)
  {
    /*first do the is connected already check*/
    if(m_istream || m_ostream){
      OAPR_THROW(oapr::runtime_error,"tcp connection is already established");
    }
    if(domain.empty()){
      OAPR_THROW(oapr::invalid_arg,"empty host name, it is required to connect");
    }
    /** if this fails it *must* throw*/
    init();
    /**@todo move this out into a stream setter option */
    oapr::pool_mgr::pool_ptr pl=oapr::pool_mgr::instance()->unique();
    std::auto_ptr<oapr::basic_socket> tmpsock(new oapr::basic_socket(pl));
    oapr::sockinfo si(pl);
    si.create(domain.c_str(),APR_UNSPEC,port,0);
    tmpsock->create(si.family(),SOCK_STREAM);
    tmpsock->connect(si);
    tmpsock->option(APR_SO_NONBLOCK,0);

    m_istream = tmpsock.get();
    m_ostream = tmpsock.get();
    m_delOstream = true;
    m_delIstream = false;
    tmpsock.release();
    return 0;
  }
  int JabSession::Authenticate(JabAuth *auth, bool sync)
  {
    /* if we aren't connected return error*/
    if(!m_ostream || !m_ostream->is_open()){
      OAPR_THROW(oapr::runtime_error,"The tcp stream is not established!");
    }
    if(m_jabAuth == BuildAuth::Auth_Succeeded){
      OAPR_THROW(oapr::runtime_error,"This session is already authenticated");
    }

    boost::scoped_ptr<BuildAuth> authBld;
    authBld.reset(auth->MakeBuilder());


    authBld->SetToken(&m_jabAuth);
    authBld->SetId(auth->Id());
    m_builders->Insert(*authBld.get());
    m_jid.Jid(auth->Jid());
    if(!m_triedAuth){
      StartThread();

      /* sending initial stream */
      std::string stream(auth->Stream());
      Send(stream);
    }

    /* we twiddle our thumbs until we get a stream:stream back*/
    while(!m_handler->getElement()){
      if(!m_istream || !m_istream->is_open()){
	OAPR_THROW(oapr::runtime_error,"Lost the tcp stream connection");
      }
      if(!m_thread.active()){
	OAPR_THROW(oapr::runtime_error,"Recv thread exited prematurely");
      }
      oapr::sleep(1);
    }
    /* getting the authentication */
    std::string sauth(auth->xml(m_handler->getElement()));
    Send(sauth);


    m_triedAuth=true;

    /* if want a sync twiddle until state changed*/
    if(sync){
      while(m_jabAuth == BuildAuth::Auth_Unknown){
	if(!m_istream || !m_istream->is_open()){
	  OAPR_THROW(oapr::runtime_error,"Lost the tcp stream connection");
	}
	if(!m_thread.active()){
	  OAPR_THROW(oapr::runtime_error,"Recv thread exited prematurely");
	}
	oapr::sleep(1);
      }
    }
    return 0;
  }
  void JabSession::Initialize(oapr::basic_io*const istrm,
			      oapr::basic_io*const ostrm,
			      const JabJid& jid)
  {
    if(m_ostream){
      OAPR_THROW(oapr::logic_error,"output stream already initialized");
    }
    if(m_istream){
      OAPR_THROW(oapr::logic_error,"input stream already initialized");
    }

    //copy the streams
    m_ostream=ostrm;
    m_delOstream=false;
    m_istream=istrm;
    m_delIstream=false;
    m_jid=jid;
    //init
    init();
    StartThread();
  }

  bool JabSession::init()
  {
    if(!m_builders){m_builders = new Builders;}
    if(!p_events){p_events = new EventChain;}
    if(!p_sigsync){p_sigsync = new oa::SigSync<oa::oa_thread>;}
    if(!m_handler){m_handler = new JabberDom;}
    //creating the sax parser and handler...
    if(!p_parser){
      p_parser = XMLReaderFactory::createXMLReader();
    }
    p_parser->setContentHandler(m_handler);
    p_parser->setDTDHandler(m_handler);
    p_parser->setEntityResolver(m_handler);
    p_parser->setErrorHandler(m_handler);
    p_parser->setLexicalHandler(m_handler);

    /**\todo make configurable*/
    m_handler->SetCache(100);
    
    return true;
  }
  void JabSession::StartThread()
  {
    //make sure we have all the info we need
    if(!m_istream || !m_ostream || !m_builders || !p_events ||
       !p_parser || !p_sigsync){
      /**@todo throw*/
    }
    //create the stream uri's
    m_outuri="http://openaether.org/stream/"+m_jid.Jid()+"/out";
    m_inuri="http://openaether.org/stream/"+m_jid.Jid()+"/in";
    RecvThread::thrdarg targs;
    targs.stream=m_istream;
    targs.builders=m_builders;
    targs.eventchain=p_events;
    targs.parser=p_parser;
    targs.sync=p_sigsync;
    targs.owner=m_jid.Jid();
    m_thread.Args(targs);
    m_thread.start();
    /* blocking until thread starts*/
    while(!m_thread.active()){
      oapr::sleep(1);
    }
  }

  int JabSession::Disconnect()
  {
    /* basically all there is to disconnecting ...
     * is to send </stream:stream>
     * but only if we are authenticated!
     */
    Shutdown();
    /*
    if(IsConnected()){
      std::string strm("</stream:stream>");
      Send(strm);

      m_thread.join();
    }



    m_tcpConnect=false;
    m_jabAuth=BuildAuth::Auth_Unknown;
    */
    return 0;
  }
  void JabSession::Shutdown()
  {
    /* several steps to shutting down completely
     * - send the closing stream if open (i.e. authentication is tried)
     * - close the istream to get the thread to die.
     * - join the recv thread
     * - destroy the xml stuff
     */
    oapr::debug::write("starting shutdown");
    oapr::debug::write("starting join thread");
    if(m_thread.state()==oapr::tsRunning){
      m_thread.signal_terminate();
    }
    oapr::debug::write("starting disconnect");

    if(IsConnected()){
      CloseStream();
      if(m_istream && m_istream->is_open()){
	m_istream->close();
      }
    }
    while(m_thread.state()!=oapr::tsTerminated){
      oapr::sleep(1);
    }
    oapr::debug::write("starting closing ostream");
    if(m_ostream && m_ostream->is_open()){
      m_ostream->close();
    }
    oapr::debug::write("starting handler kill");
    if(m_handler){
      delete m_handler;
      m_handler=0;
    }
    oapr::debug::write("ending shutdown");
  }

};//jabber ns
