/* -*- Mode: c++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 * OAjabber library
 * Contains objects to aid in Jabber client development and extend the
 * basic functionality of jabber.
 *
 * Copyright (C) 2001 Justin Kirby
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * @author Justin Kirby <justink@rochester.rr.com>
 *
 *******************************************************************/
#ifndef OAJABBER_JABSESSION_H
#define OAJABBER_JABSESSION_H

#include "xstrm/oaxstrm_exports.h" 
#include "xstrm/jabbuilders.h"
#include "xstrm/jabbldauth.h"
#include "xstrm/jabevent.h"
#include "xstrm/iojabber.h"
#include "xstrm/jabberdom.h"
#include "xstrm/jabrecvthrd.h"
namespace jabber
{
  class BuildAuth;
  class JabAuth;
  class TcpThrd;
  class OAXSTRM_API JabSession;
  ///Main session manager
  /**
   * JabSession manages the connection to the jabber
   * server. It performs several actions to extablish 
   * this connection.
   * - Opens a tcp stream
   *    A tcp stream is opened to the jabber server
   * - Instantiates several objects
   *		-# A smtcpstream for the tcp connection
   *		-# The SAXParser for incoming XML
   *		-# A TcpThrd object to manage incoming data
   * - Performs Necessary initialization
   *		-# Opens the tcp stream
   *		-# Sends the opening <stream:stream> node
   *		-# Starts a thread to handle the asynchronous incoming XML
   *              -# Authenticates based off of JabAuth
   *
   * 
   */

  class JabSession
  {
  public:
    typedef RecvThread::builders_ptr builders_ptr;
    typedef RecvThread::eventchain_ptr eventchain_ptr;
    typedef RecvThread::istream_ptr istream_ptr;
    typedef RecvThread::ostream_ptr ostream_ptr;
    typedef RecvThread::sync_ptr sync_ptr;
  public:
    JabSession();
    ///The dtor
    /**
     * Right now this simply deletes the tcp stream
     * However, since its used in both threads its
     * very doubtful as to how wise that is
     */
    virtual ~JabSession();
    
    ///The main connection and creation function
    /**
     * Establishes the connection and initializes
     * the other necessary components.
     * @param builders The list of builders to use
     * @param event The Event Chain to use for this session
     * @param pckt the authentication mechanism
     * @param prt The port to connect on (this should be moved to auth)
     * @param sync Whether to wait for authentication or continue
     * @return zero on success, non-zero on failure
     * @throws
     *  - oaexc::InvalidArgument for when the pckt arg is null.
     * This also will not catch any exceptions thrown from called functions
     * such as Connect and init
     */
    int Connect(builders_ptr builders,
                eventchain_ptr event,
                JabAuth *pckt, 
                int prt,
                bool sync=true);
    ///Establishes a tcp connection
    /**
     * Establishes a raw tcp connection to the server
     * @param domain The domain name of the server to connection to
     * @param port The port to connect on
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::Runtime Will be thrown if the tcp stream is already
     * established.
     * - oaexc::InvalidArgument Will be thrown if the host name is empty
     */
    int Connect(const std::string domain, int port);

    ///Authenticates to the connected server
    /**
     * Authenticates to the connected server.
     * @param auth JabAuth to use to authenticate.
     * @param sync Whether to wait for authentication or continue
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::Runtime Will be thrown if the tcp stream is not
     * established or this is already authenticated
     */
    int Authenticate(JabAuth *auth, bool sync=true);

    ///Initialize the session generically
    /**
     * This allows the initialization of the session
     * in a generic manner. It sets the input and output
     * stream. then starts the listening thread.
     * @note the streams have to be initialized before
     * @warning the caller is responsible for deallocation
     * of the streams
     * @param istrm pointer to input stream
     * @param ostrm pointer to output stream
     */
    void Initialize(oapr::basic_io*const istrm,
                    oapr::basic_io*const ostrm,
                    const JabJid& jid);
    ///Disconnects from the server
    /**
     * Send a </stream:stream> to the jabber end point
     * and clean up all the resources allocated during the 
     * session. With the exception of the ones needed to 
     * re-establish the session
     */
    int Disconnect();
    void Shutdown();

    ///Determine whether the session is connected
    /**
     * Return value indicates whether the session is
     * connected. This does NOT indicate whether the
     * jabber protocol is ok, just the TCP stream
     * @return true if connected, false otherwise
     */
    bool IsConnected()const{
      if(m_istream && m_istream->is_open() &&
         m_ostream && m_ostream->is_open()){
        return true;
      }else{
        return false;
      }
    }
    bool IsAuthenticated()const{
      return (m_jabAuth==BuildAuth::Auth_Succeeded);
    }
    ///Sends a packet to the jabber endpoint
    /**
     * Sends the packet off to the jabber endpoint
     * Essentially it calls pckt->xml() to get the
     * string to send over the wire.
     * @param pckt the Packt to send
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::InvalidArgument Will be thrown if the pckt argument is null
     */
    int Send(JabPacket *pckt);
    ///Send a string to the jabber endpoint
    /**
     * Simply takes the string and sends it over the
     * tcp stream. It is up to the user to make sure this
     * is valid xml, or fits in with the expectations of the
     * end point.
     * @param xml string to send
     * @return zero on success, non-zero on failure
     * @throws
     * - oaexc::InvalidArgument will be thrown if the xml argument
     * is empty
     * - oaexc::Runtime will be thrown if the tcp stream is not
     * connected
     */
    int Send(const std::string &xml);
    ///Should only be called by the thread!!
    /**
     * Do not ever ever ever call this. Or I shall thwap you
     * with a large trout!
     *
     * Upon successful building of a packet from the receiving
     * thread, the packet is sent to this session, which in turn
     * sends it off the the EvenChain internal to this session.
     * @param pckt JabPacket built from received xml
     */
    void OnEvent(JabPacket *pckt);
    ///Grabs the Builder collection
    /**
     * Grab the Builders associated with this session.
     * If one does not exist then it is created. So the 
     * caller is guaranteed to have something work with
     * @param builders the builders used in this session
     */
    void GetBuilders(builders_ptr &builders);
    ///Grabs the event chain
    /**
     * Essentially the same as GetChain except
     * using an EventChain
     * @param event the EventChain
     */
    void GetEvent(eventchain_ptr &event);
    
    ///Get the Jabber ID 
    /**
     * Returns the jabber ID associated with this session
     * @return JabJid containing the jid for this session
     */
    JabJid GetJid();
    
  private:
    ///Personal init function
    bool init();
    ///starts the thread and verifies all data is correct
    void StartThread();    
    virtual void CloseStream(){
      Send("</stream:stream>");
    }
  private:
    bool m_triedAuth;//!< whether we have been inited;
    bool m_tcpConnect;//!< whether the tcp has been established
    BuildAuth::AUTH_STATE m_jabAuth;//!< whether we have been auth'd
    JabberDom *m_handler;
    RecvThread m_thread;
    xercesc::SAX2XMLReader* p_parser;//!< the saxparser object
    sync_ptr p_sigsync;
    istream_ptr m_istream;
    ostream_ptr m_ostream;
    bool m_delIstream;
    bool m_delOstream;
    builders_ptr m_builders;
    eventchain_ptr p_events;//!< the event chain
    JabJid m_jid;
    std::string m_outuri;//!< the uri for the out going stream dom
    std::string m_inuri;//!< the uri for the in coming stream dom
  };
};//jabber ns

#endif
